/*****************************************************************************
 *
 * cgi_graph - cgi to call graphs.
 *
 * Copyright (c) 2004 Ben Clewett (bclewett@perfparse.org)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <config.h>
#include <gettext.h>
#include <locale.h>
#include "libpp_common.h"
#include "cgi_graph.h"
#include "object_users.h"

/* Internal Functions */

/* Get Max and Min for metric */
int getRange(double *dMaxRange, double *dMinRange);
void showStats();
// void getRelativeToAbsolte(int iDays, char *sTime, time_t *t0, time_t *t1);
// time_t getTime(char *sYYYYMMDDhhmmss);
// void writeDate(time_t t, char *s);
double my_abs(double d);
void getGraphCGI();

/* Module global variables */
double dMaxRange;
double dMinRange;
double dMaxIn;
double dMinIn;
int bValidRange;
char *sTitle;
char *sZoom;
int bSubmit;
float fGSigma = -1;
char sSize[10];
char sOutput[10];
char *sTimeRange, *sRelativeDays, *sRelativeTime;
int iTime;
char *sHost;
char *sService;
char *sMetric;
char sFrom[30], sTo[30];

/* Used for graph URL */
GString		*sCGI = NULL;

/* Lines */
int bValue, bSmooth, bWarn, bCrit, bStanDev;

/* An enumeration for scale */
#define SCALE_USER			0
#define SCALE_USER_EXACT	1
#define SCALE_USER_PERCENT	2
#define SCALE_RANGE			3
#define SCALE_RANGE_EXACT	4
#define SCALE_RANGE_PERCENT	5
#define SCALE_AUTO			6
int iScale = SCALE_RANGE_EXACT;




void displayGraph()
{
	char sUnit[40];
	char sTmp[128];

	sCGI = g_string_sized_new(SQL_SIZE);

	
	/* Get Unit */
	g_string_printf(s_SQL, "SELECT unit FROM perfdata_service_metric WHERE host_name = '%s'",
		sql_escape(sHost,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND service_description = '%s'",
		sql_escape(sService,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND metric = '%s'",
		sql_escape(sMetric,PERCENT_ESCAPE_CHAR));
	query(s_SQL->str);
	#ifdef USE_DB_MYSQL
	result_row = mysql_fetch_row(query_result);
	#elif defined USE_DB_POSTGRESQL
	currentRow=0;
	#endif
	strcpy(sUnit, sData(0));

	/* Graph */
	g_string_printf(sCGI, "host=%s", url_encode(sHost));
	g_string_append_printf(sCGI, "&service=%s", url_encode(sService));
	g_string_append_printf(sCGI, "&metric=%s", url_encode(sMetric));
	switch (iScale) {
	case SCALE_USER:
		g_string_append_printf(sCGI, "&forcescale=1");
		snprintf(sTmp, 40, "%f", dMinIn);
		g_string_append_printf(sCGI, "&min=%s", url_encode(sTmp));
		snprintf(sTmp, 40, "%f", dMaxIn);
		g_string_append_printf(sCGI, "&max=%s", url_encode(sTmp));
		break;
	case SCALE_USER_EXACT:
		g_string_append_printf(sCGI, "&forcescale=1&accuratescale=1");
		snprintf(sTmp, 40, "%f", dMinIn);
		g_string_append_printf(sCGI, "&min=%s", url_encode(sTmp));
		snprintf(sTmp, 40, "%f", dMaxIn);
		g_string_append_printf(sCGI, "&max=%s", url_encode(sTmp));
		break;
	case SCALE_USER_PERCENT:
		g_string_append_printf(sCGI, "&forcescale=1&accuratescale=1&percent=1");
		snprintf(sTmp, 40, "%f", dMinIn);
		g_string_append_printf(sCGI, "&min=%s", url_encode(sTmp));
		snprintf(sTmp, 40, "%f", dMaxIn);
		g_string_append_printf(sCGI, "&max=%s", url_encode(sTmp));
		break;
	case SCALE_AUTO:
		break;
	case SCALE_RANGE:
		g_string_append_printf(sCGI, "&forcescale=1");
		snprintf(sTmp, 40, "%f", dMinIn);
		g_string_append_printf(sCGI, "&min=%s", url_encode(sTmp));
		snprintf(sTmp, 40, "%f", dMaxIn);
		g_string_append_printf(sCGI, "&max=%s", url_encode(sTmp));
		break;
	case SCALE_RANGE_EXACT:
		g_string_append_printf(sCGI, "&forcescale=1&accuratescale=1");
		snprintf(sTmp, 40, "%f", dMinIn);
		g_string_append_printf(sCGI, "&min=%s", url_encode(sTmp));
		snprintf(sTmp, 40, "%f", dMaxIn);
		g_string_append_printf(sCGI, "&max=%s", url_encode(sTmp));
		break;
	case SCALE_RANGE_PERCENT:
		g_string_append_printf(sCGI, "&forcescale=1&percent=1&accuratescale=1");
		snprintf(sTmp, 40, "%f", dMinIn);
		g_string_append_printf(sCGI, "&min=%s", url_encode(sTmp));
		snprintf(sTmp, 40, "%f", dMaxIn);
		g_string_append_printf(sCGI, "&max=%s", url_encode(sTmp));
		break;
	}

	/* Lines */
	g_string_append_printf(sCGI, "&select_line=1&s_val=%d&s_smo=%d&s_war=%d&s_cri=%d&s_sta=%d",
		bValue, bSmooth, bWarn, bCrit, bStanDev);

	setlocale(LC_NUMERIC, "POSIX");
	if (fGSigma > -1) g_string_append_printf(sCGI, "&g_sigma=%f", fGSigma);
	setlocale(LC_NUMERIC, "");

	if (sUnit[0]) g_string_append_printf(sCGI, "&unit=%s", url_encode(sUnit));
	if (sSize[0]) g_string_append_printf(sCGI, "&size=%s", url_encode(sSize));
	if (sOutput[0]) g_string_append_printf(sCGI, "&output=%s", url_encode(sOutput));
	if (sTimeRange[0] != 'r') {
		g_string_append_printf(sCGI, "&from=%s", url_encode(sFrom));
		g_string_append_printf(sCGI, "&to=%s", url_encode(sTo));
	} else {
		g_string_append_printf(sCGI, "&timeperiod=r");
		g_string_append_printf(sCGI, "&reltime=%s", url_encode(sRelativeTime));
		g_string_append_printf(sCGI, "&reldays=%s", url_encode(sRelativeDays));
	}

	/* Title */
	g_string_append_printf(sCGI, "&title=%s", url_encode(sTitle));

	/* PID for use in tmp files */
	g_string_append_printf(sCGI, "&pid=%u", (unsigned int)getpid());

	/* Delete statistics files */
	sprintf(sTmp, "%s-PID-%u", FILE_STATISTICS, (unsigned int)getpid());
	remove(sTmp);
	sprintf(sTmp, "%s-STATS-%u", FILE_STATISTICS, (unsigned int)getpid());
	remove(sTmp);

	/* Actual image */
	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td align=center>\n");
	printf("<img src=\"perfchart.png?%s\" border=0>\n", sCGI->str);
	printf("</td></tr></table></p>\n");

	fflush(stdout);
	
	#ifdef SHOW_STATS
	/* Sleep a second to try and make schedular start graph */
	sleep(1);
	#endif

	/* Save: */
	if (icgi("save_graph", 0)) {

		/* Save graph */
		g_string_printf(s_SQL, "INSERT INTO perfdata_graphs (group_name, title, url) VALUES ( NULL");
		g_string_append_printf(s_SQL, ", '%s'", sql_escape(scgi("graph_name"), PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, ", '%s')", sql_escape(sCGI->str, PERCENT_ESCAPE_CHAR));
		query_no_return(s_SQL->str);

		/* Get reference */
		#ifdef USE_DB_MYSQL
		query("SELECT LAST_INSERT_ID()");
		result_row = mysql_fetch_row(query_result);
		#elif defined USE_DB_POSTGRESQL
		//query("SELECT CURRVAL('id')");
		query("SELECT CURRVAL('perfdata_graphs_id_seq')");
		currentRow=0;
		#endif

		printf("<p align=\"center\"><font color=\"#FF0000\" face=\"Arial, Helvetica\" size=2><b>");
		printf("Graph saved with ID = %d", iData(0));
		printf("</b></font></p>\n");

	}

}




void showStats()

{
	#ifndef SHOW_STATS

	return;

	#else

	
	float dMin, dMax, dSum, dSumSquare, dStanDev;
	FILE *f;
	int iTime = 0, iCount = 0;
	char sTmp[128];
	pid_t iChildPid;


	fflush(stdout);

	// Wait for the pin number file:
	sprintf(sTmp, "%s-PID-%u", FILE_STATISTICS, (unsigned int)getpid());
	iTime = 0;
	do {
		f = fopen(sTmp, "r");
		if (f) break;
		sleep(1);
	} while (iTime++ < 5);

	if (iTime >= 5) {
		// Program crashed or not started 
		// Error, this method does not work through a proxy.
		// printf("<p align=center><font face=\"Arial, Helvetica\" size=1 color=\"#FF0000\">Warning, graph is taking too long, or may have died.</font></p>\n");
		return;
	}

	fscanf(f, "%u", &iChildPid);

	fclose(f);
	remove(sTmp);


	// Wait for the stats file, then show values:
	sprintf(sTmp, "%s-STATS-%u", FILE_STATISTICS, (unsigned int)getpid());
	iTime = 0;
	do {

		// Check the child is still alive:
		if (!kill(iChildPid, 0)) {
			// Yes, still here.
		} else {
			// return, child died
			printf("<p align=center><font face=\"Arial, Helvetica\" size=1 color=\"#FF0000\">%s</font></p>\n",_("Warning, graph seems to have died."));
		return;
	}

	// See if file is written:
	f = fopen(sTmp, "r");
		if (f) break;
		sleep(1);

	} while (iTime++ < 60);

	if (iTime >= 60) {
		printf("<p align=center><font face=\"Arial, Helvetica\" size=1 color=\"#FF0000\">%s</font></p>\n",_("Warning, graph is taking too long, may have died."));
		return;
	}

	dSum = dSumSquare = dMin = dMax = dStanDev = 0;

	setlocale(LC_NUMERIC, "POSIX");
	fscanf(f, "%d %f %f %f %f", &iCount, &dSum, &dSumSquare, &dMax, &dMin);
	setlocale(LC_NUMERIC, "");
	fclose(f);

	if (iCount > 1) {
		dStanDev = (dSumSquare - (dSum * dSum / iCount)) / (iCount - 1);
		dStanDev = pow(my_abs(dStanDev), 0.5);
	} else {
		dStanDev = 0;
	}

	fclose(f);
	remove(sTmp);

	/* Stats: */
	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=2 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("<table width=\"100%%\" border=0 cellpadding=0 cellspacing=2>\n");
	printf("<tr>\n");
	printf("  <td colspan=4 align=center>");
	printf(_("%s<b>Statistics.</b><br>Sample Points: %d"), FONT, iCount);
	printf("</font></td>\n");
	printf("</tr>\n");
	if (iCount) {
		printf("<tr><td colspan=5><hr width=\"100%%\"></td></tr>\n");
		printf("<tr>");
		printf("  <td align=right>%s%s</font></td>\n", FONT,_("<b>Minimum:</b>"));
		printf("  <td align=right>%s%f</font></td>\n", FONT, dMin);
		printf("  <td align=right>%s%s</font></td>\n", FONT,_("<b>Average:</b>"));
		printf("  <td align=right>%s%f</font></td>\n", FONT, dSum / iCount);
		printf("  <td width=\"10%%\">&nbsp;</td>\n");
		printf("</tr><tr>\n");
		printf("  <td align=right>%s%s</font></td>\n", FONT,_("<b>Maximum:</b>"));
		printf("  <td align=right>%s%f</font></td>\n", FONT, dMax);
		printf("  <td align=right>%s%s</font></td>\n", FONT,_("<b>Standard&nbsp;Dev:</b>"));
		printf("  <td align=right>%s%f</font></td>\n", FONT, dStanDev);
		printf("  <td width=\"10%%\">&nbsp;</td>\n");
		printf("</tr>\n");
	}
	printf("</table>\n");
	printf("</td></tr></table></p>\n");

	fflush(stdout);

	#endif
}




void displayGraphHeader()
{

	/* Get any outstanding CGI */
	getGraphCGI();

	/* check dates */
	if (sFrom[0] <= ' ' || sTo[0] <= ' ') {
		#ifdef USE_DB_MYSQL
		query("SELECT NOW(), DATE_ADD(NOW(), INTERVAL -7 DAY)");
		result_row = mysql_fetch_row(query_result);
		#elif defined USE_DB_POSTGRESQL
		query("SELECT NOW(), NOW() - INTERVAL '7 DAY'");
		currentRow=0;
		#endif
		strcpy(sFrom, sData(1));
		strcpy(sTo, sData(0));
	}

	/* Zoom dates */
	/* Not yet commissioned. */
	/*
	if (sZoom[0] == 'l' || sZoom[0] == 'c' || sZoom[0] == 'r') {
		time_t t0, t1, t;
		// Change to absolute:
		if (sTimeRange[0] == 'r') {
			getRelativeToAbsolte(atoi(sRelativeDays), sRelativeTime, &t0, &t1);
			strcpy(sTimeRange, "a");
		} else {
			// Get dates as time:
			t0 = getTime(sFrom);
			t1 = getTime(sTo);
		}
		switch (sZoom[0]) {
		case 'l':
			// Left
			t1 = (t1 + t0) / 2;
			break;
		case 'r':
			// right
			t0 = (t1 + t0) / 2;
			break;
		case 'c':
			// center
			t = (t1 + t0) / 4;
			t0 += t;
			t1 -= t;
			break;
		}
		// Write back:
		writeDate(t0, sFrom);
		writeDate(t1, sTo);
	
	}
	*/


	/* Get Max and Min for metric */
	bValidRange = getRange(&dMaxRange, &dMinRange);

	if (iScale == SCALE_RANGE || iScale == SCALE_RANGE_PERCENT || iScale == SCALE_RANGE_EXACT) {
		if (!bValidRange) iScale = SCALE_AUTO;
	}

	/* Get the max and min values */
	switch (iScale) {
	case SCALE_USER:
	case SCALE_USER_EXACT:
	case SCALE_USER_PERCENT:
	case SCALE_AUTO:
		dMaxIn = atof(scgi("maxin"));
		dMinIn = atof(scgi("minin"));
		break;
	case SCALE_RANGE:
	case SCALE_RANGE_EXACT:
	case SCALE_RANGE_PERCENT:
		dMaxIn = dMaxRange;
		dMinIn = dMinRange;
		break;
	}

	printf("<form method=\"get\" name=\"graph\">\n");
	printf("<input type=hidden name=\"graph\" value=1>\n");
	printf("<input type=\"hidden\" name=\"host\" value=\"%s\">\n", sHost);
	printf("<input type=\"hidden\" name=\"service\" value=\"%s\">\n", sService);
	printf("<input type=\"hidden\" name=\"metric\" value=\"%s\">\n", sMetric);
	printf("<input type=\"hidden\" name=\"submit\" value=1>\n");

	/* Header */

	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");

	printf("<table width=\"100%%\" border=0 cellpadding=0 cellspacing=2>\n");
	printf("<tr>\n");
	printf("  <td colspan=6><table width=\"100%%\" border=0 cellpadding=0 cellspacing=0><tr><td>\n");
	printf("    <td width=60><img src=\"%s%s/perfparse-logo-sm.png\" border=0></td>\n", 
		(HTTP_IMAGE_PATH[0] == '/') ? "" : "/", HTTP_IMAGE_PATH);
	printf("    <td align=center>%s%s<br><br>\n", FONT,_("<b>PerfParse Metric Analysis</b>"));
	printf("      <a href=\"%s\">%s</a><br>\n", sProgName,_("Return to main menu"));
	printf("      <a href=\"%s?all_bin=1\">%s</a>\n", sProgName,_("Select new Metric"));
	printf("    </td>\n");
	printf("    <td width=60 align=right>%sv%s</td>\n", FONT_SMALL, VERSION);
	printf("  </td></tr></table></td>\n");
	printf("</tr>\n");

	printf("<tr><td colspan=6><hr width=\"100%%\"></td></tr>\n");

	// Host Metric and Service:
	printf("<tr>\n");
	printf("  <td width=\"16%%\" align=right>%s<b>%s:</b></font></td>\n", FONT,_("Host"));
	printf("  <td width=\"16%%\">%s&nbsp;%s</font></td>\n", FONT, sHost);
	printf("  <td width=\"16%%\" align=right>%s<b>%s:</b></font></td>\n", FONT,_("Service"));
	printf("  <td width=\"16%%\">%s&nbsp;%s</font></td>\n", FONT, sService);
	printf("  <td width=\"16%%\" align=right>%s<b>%s:</b></font></td>\n", FONT,_("Metric"));
	printf("  <td width=\"16%%\">%s&nbsp;%s</font></td>\n", FONT, sMetric);
	printf("</tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\"></td></tr>\n");

	/* Range */
	printf("<tr>\n");
	printf("  <td align=right>%s%s:</font></td>\n", FONT,_("Scale"));
	printf("  <td>\n");
	printf("    <select name=\"scale\">\n");
	printf("    <option value=\"%d\"%s>%s:\n",
		SCALE_USER_EXACT,		(iScale == SCALE_USER_EXACT) ? " selected" : "",_("User"));
	printf("    <option value=\"%d\"%s>%s:\n",
		SCALE_USER,				(iScale == SCALE_USER) ? " selected" : "",_("User, Rounded"));
	printf("    <option value=\"%d\"%s>%s:\n",
		SCALE_USER_PERCENT,		(iScale == SCALE_USER_PERCENT) ? " selected" : "",_("User, as Percent"));
	if (bValidRange) {
		printf("    <option value=\"%d\"%s>%s\n",
			SCALE_RANGE_EXACT,		(iScale == SCALE_RANGE_EXACT) ? " selected" : "",_("From Plugin."));
		printf("    <option value=\"%d\"%s>%s\n",
			SCALE_RANGE,			(iScale == SCALE_RANGE) ? " selected" : "",_("From Plugin, Rounded."));
		printf("    <option value=\"%d\"%s>%s\n",
			SCALE_RANGE_PERCENT,	(iScale == SCALE_RANGE_PERCENT) ? " selected" : "",_("From Plugin, as Percent"));
	}
	printf("    <option value=\"%d\"%s>%s\n",
		SCALE_AUTO,				(iScale == SCALE_AUTO) ? " selected" : "",_("Automatic."));
	printf("    </select>\n");
	printf("  </td>\n");

	/* Range Values */
	printf("  <td align=right>%s\n", FONT);
	printf("    %s\n",_("User Min:"));
	printf("  </font></td>\n");
	printf("  <td>\n");
	setlocale(LC_NUMERIC, "POSIX");
	printf("    <input type=text name=\"minin\" value=\"%f\" size=12 maxlength=12>\n", dMinIn);
	setlocale(LC_NUMERIC, "");
	printf("  </td>\n");
	printf("  <td align=right>%s\n", FONT);
	printf("    %s\n",_("Max:"));
	printf("  </font></td>\n");
	printf("  <td>\n");
	setlocale(LC_NUMERIC, "POSIX");
	printf("    <input type=text name=\"maxin\" value=\"%f\" size=12 maxlength=12>\n", dMaxIn);
	setlocale(LC_NUMERIC, "");
	printf("  </td>\n");
	printf("</tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\"></td></tr>\n");

	/* Time Range  Type*/
	printf("<tr>\n");
	printf("  <td align=right rowspan=2>%s%s</font></td>\n", FONT,_("Period:"));
	printf("  <td valign=middle>%s", FONT);
	printf("    <input type=radio name=\"timerange\" value=\"r\" onclick=\"isAbsRelVisible()\"%s>%s",
		(sTimeRange[0] == 'r') ? " checked" : "",_("Relative"));
	printf("  </font></td>\n");

	/* Relative Dates */
	printf("  <td align=right id=\"Rel1\">%s%s</font></td>\n", FONT,_("Rel&nbsp;Period:"));
	printf("  <td valign=middle id=\"Rel2\">%s", FONT);
	printf("    <input type=text name=\"relativedays\" value=\"%s\" size=2 maxlength=4>&nbsp;%s", sRelativeDays,_("Days"));
	printf("  </font></td>\n");
	printf("  <td align=right id=\"Rel3\">%s\n", FONT);
	printf("    %s\n",_("And Hours:"));
	printf("  </font></td>\n");
	printf("  <td id=\"Rel4\">%s\n", FONT);
	printf("    <input type=text name=\"relativetime\" value=\"%s\" size=5 maxlength=5>&nbsp;%s", sRelativeTime,_("HH:MM"));
	printf("  </font></td>\n");
	printf("</tr>\n");


	/* Absolute Dates */
	printf("<tr>\n");
	printf("  <td>%s\n", FONT);
	printf("    <input type=radio name=\"timerange\" value=\"a\" onclick=\"isAbsRelVisible()\"%s>%s",
		(sTimeRange[0] == 'a' || sTimeRange[0] == 0) ? " checked" : "",_("Absolute"));
	printf("  </font></td>\n");
	printf("  <td align=right id=\"Abs1\">%s%s</font></td>\n", FONT,_("Abs&nbsp;From:"));
	printf("  <td id=\"Abs2\"><input type=text name=\"from\" value=\"%s\" size=17 maxlength=20></td>\n", sFrom);
	printf("  <td align=right id=\"Abs3\">%s%s</font></td>\n", FONT,_("Abs&nbsp;To:"));
	printf("  <td id=\"Abs4\"><input type=text name=\"to\" value=\"%s\" size=17 maxlength=20></td>\n", sTo);
	printf("</tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\" height=4></td></tr>\n");

	/* Size and Type */
	printf("<tr>\n");
	printf("  <td align=right>%sOutput:</font></td>\n", FONT);
	printf("  <td><select name=output>\n");
	printf("     <option value=\"Histogram\"%s>%s\n", (!strcmp(sOutput, "Histogram") || !sOutput[0]) ? " selected" : "",_("Histogram"));
	printf("     <option value=\"Average\"%s>%s\n", (!strcmp(sOutput, "Average")) ? " selected" : "",_("Daily Average"));
//	printf("     <option value=\"XML\"%s>%s\n", (!strcmp(sOutput, "XML")) ? " selected" : "",_("CSV"));
//	printf("     <option value=\"CSV\"%s>%s\n", (!strcmp(sOutput, "CSV")) ? " selected" : "",_("CSV"));
	printf("     <option value=\"Diff\"%s>%s\n", (!strcmp(sOutput, "Diff")) ? " selected" : "",_("Differential Rate"));
	printf("     <option value=\"Int\"%s>%s\n", (!strcmp(sOutput, "Int")) ? " selected" : "",_("Integral Sum"));
	printf("  </td>\n");
	printf("  <td colspan=2>&nbsp;</td>\n");
	printf("  <td align=right>%s%s</font></td>\n", FONT,_("Size:"));
	printf("  <td><select name=size>\n");
	printf("     <option value=\"400x200\"%s>Small\n", (!strcmp(sSize, "400x200")) ? " selected" : "");
	printf("     <option value=\"585x300\"%s>Normal\n", (!strcmp(sSize, "585x300") || !sSize[0]) ? " selected" : "");
	printf("     <option value=\"1000x600\"%s>Large\n", (!strcmp(sSize, "1000x600")) ? " selected" : "");
	printf("  </td>\n");
	printf("</tr>\n");

	// Show what:
	printf("<tr>\n");
	printf("  <td align=right>%s%s</font></td>\n", FONT, _("Plot:"));
	printf("  <td colspan=5><font face=\"Arial, Helvetica\" size=\"2\">\n");
	printf("  <input type=checkbox name=\"s_val\" value=\"1\"%s>&nbsp;%s&nbsp;&nbsp;",
		(bValue) ? " checked" : "",_("Value"));
	printf("  <input type=checkbox name=\"s_smo\" value=\"1\"%s>&nbsp;%s&nbsp;&nbsp;",
		(bSmooth) ? " checked" : "",_("Smooth*"));
	printf("  <input type=checkbox name=\"s_war\" value=\"1\"%s>&nbsp;%s&nbsp;&nbsp;",
		(bWarn) ? " checked" : "",_("Warning"));
	printf("  <input type=checkbox name=\"s_cri\" value=\"1\"%s>&nbsp;%s&nbsp;&nbsp;",
		(bCrit) ? " checked" : "",_("Critical"));
	printf("  <input type=checkbox name=\"s_sta\" value=\"1\"%s>&nbsp;%s&nbsp;&nbsp;",
		(bStanDev) ? " checked" : "",_("Stan.&nbsp;Dev*"));
	printf("  </font>\n");
	printf("  <font face=\"Arial, Helvetica\" size=\"1\">(%s)</font>\n",_("*&nbsp;Where&nbsp;Used"));
	printf("</td>\n");
	printf("</tr>\n");


	printf("</table>\n");

	printf("</td></tr></table></p>\n");
	
	/* Because we are not in a table, a flush now will force first half of page to be displayed */
	fflush(stdout);

	/* Now the graph */
	displayGraph();

	/* Button Bar.  Do not display earlier, page still drawing. */
	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");

	printf("<table width=\"100%%\" border=0 cellpadding=0 cellspacing=5>\n");
	printf("<tr>\n");
	printf("  <td width=\"50%%\">\n");
	/* Zoom.  Not yet commissioned. */
	/*
	printf("<table border=0 cellpadding=0 cellspacing=2><tr>\n");
	printf("  <td rowspan=2>%sZoom:</font></td>\n", FONT);
	printf("  <td>%s<input type=\"radio\" name=\"zoom\" value=\"\" checked>None</font></td>\n", FONT);
	printf("  <td>%s<input type=\"radio\" name=\"zoom\" value=\"c\">Center</font></td>\n", FONT);
	printf("</tr><tr>\n");
	printf("  <td>%s<input type=\"radio\" name=\"zoom\" value=\"l\">Left</font></td>\n", FONT);
	printf("  <td>%s<input type=\"radio\" name=\"zoom\" value=\"r\">Right</font></td>\n", FONT);
	printf("</tr></table>\n");
	*/
	printf("  </td>\n");
	printf("  <td><input type=submit value=\"%s\"></td>\n",_("Draw Graph"));
	printf("  <td width=\"50%%\">&nbsp;</td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	printf("</td></tr></table></p>\n");

	/* Advanced Features */
	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");

	printf("<table width=\"100%%\" border=0 cellpadding=0 cellspacing=2>\n");
	printf("<tr>\n");
	printf("  <td colspan=4 align=center>%s%s</font></td>\n", FONT,_("<b>Advanced Options.</b>"));
	printf("</tr>\n");

	// Title:
	if (!sTitle[0]) {
		g_string_printf(s_SQL, "%s '%s'   %s '%s'   %s '%s'", _("Host"), sHost, _("Service"), sService, _("Metric"), sMetric);
		free(sTitle);
		sTitle = strdup(s_SQL->str);
	}
	printf("<tr>\n");
	printf("  <td colspan=2 align=right>%s\n", FONT);
	printf("    %s\n", _("Graph Legend:"));
	printf("  </font></td>\n");
	printf("  <td colspan=2>\n");
	printf("    <input type=\"text\" name=\"title\" value=\"%s\" size=60 maxlength=80>\n", sTitle);
	printf("  </td>\n");
	printf("</tr>\n");

	/* Range Values */
	printf("<tr>\n");
	printf("  <td align=right colspan=2>%s\n", FONT);
	printf("    %s\n",_("Smooth Plot: Gaussian Average: Width (pixels):"));
	printf("  </font></td>\n");
	printf("  <td colspan=2><nobr>\n");
	setlocale(LC_NUMERIC, "POSIX");
	printf("    <input type=\"text\" name=\"g_sigma\" value=\"%.1f\" size=5 maxlength=5>\n", fGSigma);
	setlocale(LC_NUMERIC, ""); 
	printf("    %s%s</font>\n", FONT_SMALL,_("(Large = Slow, see <a href=\"http://perfparse.sourceforge.net/faq.php#gaussian\">FAQ</a>)"));
	printf("  </td>\n");
	printf("</tr>\n");

	/* Store graph */
	printf("<tr>\n");
	printf("  <td align=right colspan=2>%s\n", FONT);
	printf("    %s\n",_("Save graph for future reference:"));
	printf("  </font></td>\n");
	printf("  <td><input type=checkbox name=\"save_graph\" value=\"1\"></td>\n");
	printf("  <td align=right>%s<nobr>\n", FONT);
	printf("    %s\n",_("Save with Title:"));
	printf("    <input type=\"text\" name=\"graph_name\" value=\"%s\" size=40 maxlength=80>\n", sTitle);
	printf("  </font></td>\n");
	printf("</tr>\n");

	printf("</table>\n");
	printf("</td></tr></table></p>\n");

	printf("</form>\n");

	// Stats:
	showStats();

	/* Show URL so that can be copied and embedded on user's open site */
	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("<table width=\"100%%\" border=0 cellpadding=0 cellspacing=5>\n");
	printf("<tr>\n");
	printf("  <td align=center>%s%s<b></b></font></td>\n", FONT,_("URL to create this graph:"));
	printf("</tr>\n");
	printf("<tr>\n");
	printf("  <td><font face=\"Courier\" size=1>%s/perfchart.png?%s</td>\n", getPath(getenv("REQUEST_URI")), sCGI->str);
	printf("</tr>\n");
	printf("</table>\n");
	printf("</td></tr></table></p>\n");

	footer();

	exit(0);

}


/* Get range of metric */
int getRange(double *dMaxRangeLocal, double *dMinRangeLocal)
{

	
	g_string_printf(s_SQL, "SELECT value_max, value_min FROM perfdata_service_metric WHERE host_name = '%s'",
		sql_escape(sHost,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND service_description = '%s'",
		sql_escape(sService,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND metric = '%s'",
		sql_escape(sMetric,PERCENT_ESCAPE_CHAR));
	query(s_SQL->str);
	#ifdef USE_DB_MYSQL
	result_row = mysql_fetch_row(query_result);
	#elif defined USE_DB_POSTGRESQL
	currentRow=0;
	#endif

	if (isNull(0) || isNull(1)) {
		*dMaxRangeLocal = 0;
		*dMinRangeLocal = 0;
		return FALSE;
	}

	*dMaxRangeLocal = atof(sData(0));
	*dMinRangeLocal = atof(sData(1));

	if (*dMaxRangeLocal <= *dMinRangeLocal) {
		*dMaxRangeLocal = 0;
		*dMinRangeLocal = 0;
		return FALSE;
	}

	return TRUE;

}



/*
time_t getTime(char *sYYYYMMDDhhmmss)
// Format 'YYYY-MM-DD hh:mm:ss' -> time_t
{
	struct tm recTime;
	static int iOffset = -1;
	time_t tOut;
	char s[5];

	if (strlen(sYYYYMMDDhhmmss) < 18) {
		int i;
		for (i = strlen(sYYYYMMDDhhmmss) + 1; i < 19; i++) sYYYYMMDDhhmmss[i] = 0;
	}

	// Year:
	s[0] = sYYYYMMDDhhmmss[0];
	s[1] = sYYYYMMDDhhmmss[1];
	s[2] = sYYYYMMDDhhmmss[2];
	s[3] = sYYYYMMDDhhmmss[3];
	s[4] = 0;
	recTime.tm_year = atoi(s) - 1900;

	// Month:
	s[0] = sYYYYMMDDhhmmss[5];
	s[1] = sYYYYMMDDhhmmss[6];
	s[2] = 0;
	recTime.tm_mon = atoi(s) - 1;

	// Month Day:
	s[0] = sYYYYMMDDhhmmss[8];
	s[1] = sYYYYMMDDhhmmss[9];
	s[2] = 0;
	recTime.tm_mday = atoi(s);

	// Hours:
	s[0] = sYYYYMMDDhhmmss[11];
	s[1] = sYYYYMMDDhhmmss[12];
	s[2] = 0;
	recTime.tm_hour = atoi(s);

	// Minuts:
	s[0] = sYYYYMMDDhhmmss[14];
	s[1] = sYYYYMMDDhhmmss[15];
	s[2] = 0;
	recTime.tm_min = atoi(s);

	// Seconds:
	s[0] = sYYYYMMDDhhmmss[17];
	s[1] = sYYYYMMDDhhmmss[18];
	s[2] = 0;
	recTime.tm_sec = atoi(s);
	recTime.tm_isdst = 0;

	tOut = mktime(&recTime);

	if (iOffset == -1) {
		struct tm *r;
		r = localtime(&tOut);
		r->tm_isdst = 0;
		iOffset = (int)mktime(r) - tOut;
	}

	return tOut - iOffset;
	
}


void writeDate(time_t t, char *s)
{
	struct tm *lt;

	lt = localtime(&t);
	sprintf(s, "%04d-%02d-%02d %02d:%02d:%02d",
		lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min, lt->tm_sec);

}


*/


double my_abs(double d)
{
	if (d < 0) d *= -1;
	return d;
}


void getGraphCGI()
{

	
	sHost = strdup(scgi("host"));
	sService = strdup(scgi("service"));
	sMetric = strdup(scgi("metric"));

	strcpy(sFrom, scgi("from"));
	strcpy(sTo, scgi("to"));

	strcpy(sSize, scgi("size"));
	
	strcpy(sOutput, scgi("output"));

 	sTimeRange = strdup(scgi("timerange"));
	if (!sTimeRange[0]) sTimeRange = "r";

	sRelativeDays = strdup(scgi("relativedays"));
	sRelativeTime = strdup(scgi("relativetime"));

	if (!sRelativeDays[0]) strcpy(sRelativeDays, "1");
	if (!sRelativeTime[0]) strcpy(sRelativeTime, "00:00");

	/* Time period and type */
	if (sTimeRange[0] == 'r') {

		int iHour = 0, iMin = 0;
		char sHour[10] = "";
		sscanf(sRelativeTime, "%[^:]:%d", sHour, &iMin);
		iHour = atoi(sHour);
		iTime = atoi(sRelativeDays);
		iTime *= 3600 * 24;
		iTime += iHour * 3600 + iMin * 60;

		getTimeRange(iTime, sFrom, sTo);

	} else {

		iTime = 0;

	}

	
	// Page submitted from it's self:
	bSubmit = icgi("submit", 0);

	// Scale options:
	iScale = icgi("scale", SCALE_RANGE_EXACT);

	// Title:
	sTitle = strdup(scgi("title"));

	// Zoom: (Not yet commissioned)
	sZoom = strdup(scgi("zoom"));

	// What lines to plot:
	if (bSubmit) {
		bValue = icgi("s_val", 0);
		bSmooth = icgi("s_smo", 0);
		bWarn = icgi("s_war", 0);
		bCrit = icgi("s_cri", 0);
		bStanDev = icgi("s_sta", 0);
	} else {
		bValue = bWarn = bCrit = bStanDev = TRUE;
		bSmooth = FALSE;
	}

	// Gaussian Sigma:
	fGSigma = atof(scgi("g_sigma"));
	if (fGSigma <= 0) fGSigma = 14;


}


void displaySavedGraphs()
{
	int bHeader = FALSE;
	char sColour[7] = "FFFFFF";
	char *s;
	
	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	sCGI = g_string_sized_new(SQL_SIZE);

	showHeader(_("Stored Graphs"), "");

	selectGroup("saved_graphs", TRUE);

	if (!user.sGroupName[0]) return;

	g_string_printf(sCGI, "SELECT id, title FROM perfdata_graphs");
	if (strcmp(user.sGroupName, "**ALL**")) {
		g_string_append_printf(sCGI, " WHERE group_name = '%s'", 
			sql_escape(user.sGroupName, PERCENT_ESCAPE_CHAR));
	}
	g_string_append_printf(sCGI, " %s", getDecInc("ORDER BY id"));
	query(sCGI->str);

	// Define URL:
	g_string_printf(sCGI, "%s?saved_graphs=1", sProgName);

	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow = 0 ;currentRow<numberOfRows; currentRow++) {
	#endif

		if (!bHeader) {

			printf("<p align=center><table width=\"600\" border=1 cellspacing=0 cellpadding=2>\n");
			fflush(stdout);

			printf("<tr>\n");

			printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><nobr>%s", "FFFFCC", FONT);
			displayDecInc("id", sCGI->str, "id");
			printf(_("ID\n"));
			printf("  </b></font></td>\n");

			printf("  <td bgcolor=\"#%s\" align=center><nobr>%s", "FFFFCC", FONT);
			displayDecInc("title", sCGI->str, "id");
			printf(_("Title\n"));
			printf("  </b></font></td>\n");

			printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><nobr>%s", "FFFFCC", FONT);
			printf(_("View\n"));
			printf("  </b></font></td>\n");

			printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><nobr>%s", "FFFFCC", FONT);
			printf(_("Edit\n"));
			printf("  </b></font></td>\n");

			printf("</tr>\n");

			bHeader = TRUE;
		}

		printf("<tr>\n");

		/* ID */
		printf("  <td bgcolor=\"#%s\">%s&nbsp;%s&nbsp;</font></td>\n", sColour, FONT, sData(0));

		/* Title */
		printf("  <td bgcolor=\"#%s\">%s&nbsp;%s&nbsp;</font></td>\n", sColour, FONT, sData(1));

		/* View */
		printf("  <td bgcolor=\"#%s\" align=center>%s&nbsp;", sColour, FONT);
		printf("<a href=\"%s?view_saved_graph=1&view=%s", sProgName, sData(0));
		s = scgi("dec");
		if (s[0]) printf("&dec=%s", s);
		s = scgi("inc");
		if (s[0]) printf("&inc=%s", s);
		printf("\">%s</a>&nbsp;</font></td>\n", _("View"));

		/* Edit */
		printf("  <td bgcolor=\"#%s\" align=center>%s&nbsp;", sColour, FONT);
		printf("<a href=\"%s?edit_saved_graph=1&view=%s", sProgName, sData(0));
		s = scgi("dec");
		if (s[0]) printf("&dec=%s", s);
		s = scgi("inc");
		if (s[0]) printf("&inc=%s", s);
		printf("\">%s</a>&nbsp;</font></td>\n",_("Edit"));

		printf("</tr>\n");


	}

	if (bHeader) printf("</table></p>\n");

}


void diaplayEditSavedGraph()
{
	char sExtra[255];
	char *sDec, *sInc;
	int iID = 0;
	char *sTitle;
	char *sURL;
	char *sDescript;
	char *sGraphGroup;
	int  iRefreshTime;

	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	sCGI = g_string_sized_new(SQL_SIZE);

	iID = icgi("view", 0);

	/* Header stuff */
	sDec = strdup(scgi("dec"));
	sInc = strdup(scgi("inc"));
	sprintf(sExtra, "<a href=\"%s?saved_graphs=1&dec=%s&inc=%s\">%s</a>&nbsp;&nbsp;<a href=\"%s?view_saved_graph=1&dec=%s&inc=%s&view=%d\">%s</a>", 
			sProgName, sDec, sInc, _("Return to Graph List"),
		sProgName, sDec, sInc, iID, _("View Graph"));


	showHeader(_("Edit Stored Graph"), sExtra);

	/* incomming data */
	if (icgi("submit", FALSE)) {


		int bDelete;
		bDelete = icgi("delete", FALSE);
		if (bDelete) {
			g_string_printf(sCGI, "DELETE FROM perfdata_graphs WHERE id = %d", iID);
			query_no_return(sCGI->str);
			printf("<p align=center><b><font color=\"#FF0000\">%s", FONT);
			printf(_("Record %d Deleted."), iID);
			printf("</font></b></font></p>\n");
			return;
		}

		iRefreshTime = icgi("refresh_time", 0);
		sTitle = strdup(scgi("title"));
		sDescript = strdup(scgi("descript"));
		sURL = strdup(scgi("url"));
		sGraphGroup = strdup(scgi("graph_group"));

		g_string_printf(sCGI, "UPDATE perfdata_graphs SET title = '%s'", 
			sql_escape(sTitle, PERCENT_ESCAPE_CHAR));
		if (sGraphGroup[0]) {
			g_string_append_printf(sCGI, ", group_name = '%s'",
				sql_escape(sGraphGroup, PERCENT_ESCAPE_CHAR));
		} else {
			g_string_append_printf(sCGI, ", group_name = NULL");
		}
		g_string_append_printf(sCGI, ", descript = '%s'",
			sql_escape(sDescript, PERCENT_ESCAPE_CHAR));
		g_string_append_printf(sCGI, ", url = '%s'",
			sql_escape(sURL, PERCENT_ESCAPE_CHAR));
		g_string_append_printf(sCGI, ", refresh_time = '%d'", iRefreshTime);
		g_string_append_printf(sCGI, " WHERE id = %d", iID);
		query_no_return(sCGI->str);

		printf("<p align=center><b><font color=\"#FF0000\">%s",FONT);
		printf(_("Record %d Saved."), iID);
		printf("</font></b></font></p>\n");

	}


	/* Form stuff */
	printf("<form method=\"get\">\n");
	printf("<input type=hidden name=\"submit\" value=\"1\">\n");
	printf("<input type=hidden name=\"edit_saved_graph\" value=\"1\">\n");
	printf("<input type=hidden name=\"view\" value=\"%d\">\n", iID);
	printf("<input type=hidden name=\"inc\" value=\"%s\">\n", sInc);
	printf("<input type=hidden name=\"dec\" value=\"%s\">\n", sDec);

	/* Get fields */
	g_string_printf(sCGI, "SELECT title, group_name, url, descript, refresh_time FROM perfdata_graphs WHERE id = %d", iID);
	query(sCGI->str);
	#ifdef USE_DB_MYSQL
	result_row = mysql_fetch_row(query_result);
	if (!result_row) {
	#elif defined USE_DB_POSTGRESQL
	currentRow = 0;
	if (rows()<=0) {
	#endif
	
		printf("<p align=center><b><font color=\"#FF0000\">%s", FONT);
		printf(_("Record %d not found?"), iID);
		printf("</font></b></font></p>\n");
		return;
	}

	sTitle = strdup(sData(0));
	sGraphGroup = strdup(sData(1));
	sURL = strdup(sData(2));
	sDescript = strdup(sData(3));
	iRefreshTime = iData(4);
	
	printf("<p align=center><table width=\"600\" border=\"1\" cellspacing=\"0\" cellpadding=\"5\" bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"2\">\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("ID:"));
	printf("    <td>%s%d</font></td>\n", FONT, iID);
	printf("  </tr>\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("Title:"));
	printf("    <td><input type=\"text\" name=\"title\" size=50 maxlength=80 value=\"%s\"></td>\n", sTitle);
	printf("  </tr>\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("Refresh&nbsp;Time:"));
	printf("    <td><input type=\"text\" name=\"refresh_time\" size=5 maxlength=8 value=\"%d\">\n", iRefreshTime);
	printf("      %s%s\n", FONT, _("Minutes. (0 = Dissabled.)"));
	printf("    </td>\n");
	printf("  </tr>\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("Delete&nbsp;Graph:"));
	printf("    <td><input type=checkbox name=\"delete\" value=\"1\"</td>\n");
	printf("  </tr>\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("Group:"));
	printf("    <td>\n");
	printf("      <select name=\"graph_group\">\n");
	printf("      <option value=\"\">&lt;%s&gt;\n",_("None"));
	query("SELECT group_name FROM perfdata_host_group");
	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
		printf("      <option%s>%s", (!strcmp(sData(0), sGraphGroup)) ? " selected" : "", sData(0));
	}
	#elif defined USE_DB_POSTGRESQL
	numberOfRows=PQntuples(query_result);
	for (currentRow=0;currentRow<numberOfRows;currentRow++){
		printf("      <option%s>%s", (!strcmp(sData(0), sGraphGroup)) ? " selected" : "", sData(0));
	}
	#endif
	
	printf("    </td>\n");
	printf("  </tr>\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("Description:"));
	printf("    <td><textarea wrap=\"virtual\" cols=\"50\" rows=\"5\" name=\"descript\" method=\"get\">%s</textarea>\n", sDescript);
	printf("  </tr>\n");

	printf("  <tr>\n");
	printf("    <td width=\"30%%\" align=\"right\" valign=\"top\">%s<b>%s</b></font></td>\n", FONT,_("URL:"));
	printf("    <td><input type=\"text\" name=\"url\" size=50 value=\"%s\"></td>\n", sURL);
	printf("  </tr>\n");

	printf("</table>\n");
	printf("</td></tt></table></p>\n");


	printf("<p align=center><input type=\"submit\" value=\"%s\"></p></form>\n", _("Save"));

}


void diaplaySelectedSavedGraph()
{
	int iID = 0;
	char sExtra[255];
	char *sDec, *sInc;
	int  iRefreshTime;

	header_meta();

	sDec = strdup(scgi("dec"));
	sInc = strdup(scgi("inc"));
	iID = icgi("view", 0);

	/* Get */
	sCGI = g_string_sized_new(SQL_SIZE);
	g_string_printf(sCGI, "SELECT url, descript, title, refresh_time FROM perfdata_graphs WHERE id = %d", iID);
	query(sCGI->str);
	#ifdef USE_DB_MYSQL
	result_row = mysql_fetch_row(query_result);
	#elif defined USE_DB_POSTGRESQL
	currentRow = 0;
	#endif
	iRefreshTime = iData(3);

	/* This calls the little used refresh time header: */
	header_refresh(iRefreshTime * 60);

	sprintf(sExtra, "<a href=\"%s?saved_graphs=1&dec=%s&inc=%s\">%s</a>&nbsp;&nbsp;<a href=\"%s?edit_saved_graph=1&dec=%s&inc=%s&view=%d\">%s</a>\n", 
		sProgName, sDec, sInc, _("Return to Graph List"),
		sProgName, sDec, sInc, iID, _("Edit Graph"));

	showHeader(_("View Stored Graph"), sExtra);

	/* ID */
	printf("<p align=center><table width=\"600\" border=1 cellpadding=2 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td align=center>%s\n", FONT);
	printf(_("<b>Graph:</b>&nbsp;&nbsp;%d&nbsp;&nbsp;<b>%s:</b>&nbsp;&nbsp;%s\n"), iID, _("Title"), sData(2));
	if (iRefreshTime) {
		printf("<br><b>%s</b>: %d:%02d", _("Refresh Time"), iRefreshTime / 60, iRefreshTime % 60);
	}
	printf("</font></td></tr></table></p>\n");

	/* Show */
	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td align=center>\n");
	printf("<img src=\"perfchart.png?%s\" border=0>\n", sData(0));
		printf("</td></tr></table></p>\n");

	/* Descript */
	if (sData(1)[0]) {
		printf("<p align=center><table width=\"600\" border=1 cellpadding=2 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td%s>%s\n", 
			(strlen(sData(1)) < 200) ? " align=center" : "", FONT);
		printf("%s\n", sData(1));
		printf("</font></td></tr></table></p>\n");
	}

}


