/*****************************************************************************
 *
 * perfgant.c - Displays a gant chart of state changes
 *
 * Copyright (c) 2004 Ben Clewett (bclewett@perfparse.org)
 * Last Modified:   2004-07-22
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <gd.h>
#include <gdfonts.h>
#include <glib.h>

#include "common.h"
#include "dbms.h"
#include <libpp_common.h>
#include "cgi.h"

#include <config.h>
#include <gettext.h>
#include <locale.h>

// #define DEBUG

#define DAY1 (3600 * 24)

/* Don't change these please */

gdImagePtr gant = 0;

void getCGI();
void doSQL();

/* CGI in */
char *sHost;
char *sService;
char *sFrom, *sTo;

time_t tFrom, tTo, tFromMidnight, tToMidnight;
int iDays;

int iWidth = 600, iHeight = 300;
const int iBorder = 30;
const int iSideBorder = 50;
const int iRowHeight = 20;
const int iRowGap = 10;

int black, white, blue, lblue, red, grey0, grey1, grey2, yellow;
int iStateColour[4];

void header();
void print_failure_image(FILE *f, char *sMsg);
void print_sql_failure_image(char*file, int line, const char *sql, const char *error);
int iYear, iMonth, iDay;
void initChart();
void drawGant();
void drawHeadings();
time_t getTime(char *sSQLDate);
void drawBox(time_t t0, time_t t1, int iState);
int getY(time_t t);
int getX(time_t t);
int getDay(time_t t);
void drawScale();

FILE *f;

int main(int argc, char *argv[])
{

        char *argvInt[1];
	int   argcInt = 1;

	int options[] = {
		CONFIG_ITEM_ID_ERROR_LOG,
		CONFIG_ITEM_ID_ERROR_LOG_ROTATE,
		CONFIG_ITEM_ID_DB_USER,
		CONFIG_ITEM_ID_DB_PWD,
		CONFIG_ITEM_ID_DB_DATABASE,
		CONFIG_ITEM_ID_DB_HOST,
		CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY,
		CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_HOST_GROUP,
		CONFIG_ITEM_ID_DUMMY_HOSTNAME,
		CONFIG_ITEM_ID_NB
	};

	printf("content-type: image/png\n\n");

	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
	
#ifdef DEBUG
	f = fopen("/tmp/gant.log", "w");
#endif
	
	argvInt[0] = argv[0];
	argcInt = 1;

	/* Get settings */
	config_parse_config(argcInt,argvInt,"perfgant.png\n\n","\n",options, "stderr");

	/* Get CGI variables */
	getCGI();

	/* Init the chart */
	initChart();

	/* Handle SQL */
	doSQL();

	/* Fill area with a grey background */
	gdImageFilledRectangle(gant, iSideBorder, iBorder, iWidth - iSideBorder, iHeight - iBorder, grey0);

	/* Draw the scale */
	drawScale();

	/* Draw Gant */
	drawGant();

	/* Borders */
	gdImageRectangle(gant, 0, 0, iWidth - 1, iHeight - 1, black);
	gdImageRectangle(gant, iSideBorder, iBorder, iWidth - iSideBorder, iHeight - iBorder, black);

	/* headings and key */
	drawHeadings();

	/* Splurge */
	gdImagePng(gant, stdout);

	exit(0);
}

void drawScale()
{ 
	double d;
	int i;
	char s[20];
	struct tm *r;
	time_t t;

	for (i = 0; i < 25; i++) {
		d = (double)i * 3600;
		d /= DAY1;
		d *= (iWidth - iSideBorder * 2);
		d += iSideBorder;
		if (i % 2 == 0) {
			gdImageLine(gant, (int)d, iBorder, (int)d, iHeight - iBorder, grey2);
			sprintf(s, "%02d:00", i);
			gdImageString(gant, gdFontSmall, (int)d - 15, iBorder - 13, s, black);
			gdImageString(gant, gdFontSmall, (int)d - 15, iHeight - 30, s, black);
		} else {
			gdImageLine(gant, (int)d, iBorder, (int)d, iHeight - iBorder, grey1);
		}

	}

	for (i = 0; i < iDays; i++) {
		t = i * DAY1 + tFromMidnight;
		r = localtime(&t);
//		sprintf(s, "%04d-%02d-%02d", r->tm_year + 1900, r->tm_mon + 1, r->tm_mday);
		sprintf(s, "%02d-%02d", r->tm_mon + 1, r->tm_mday);
		gdImageString(gant, gdFontSmall, 8, i * (iRowHeight + iRowGap) + iBorder + 12, s, black);
		switch (r->tm_wday) {
		case 0:		strncpy(s, _("Sun"),4);	break;
		case 1:		strncpy(s, _("Mon"),4);	break;
		case 2:		strncpy(s, _("Tue"),4);	break;
		case 3:		strncpy(s, _("Wed"),4);	break;
		case 4:		strncpy(s, _("Thu"),4);	break;
		case 5:		strncpy(s, _("Fri"),4);	break;
		case 6:		strncpy(s, _("Sat"),4);	break;
		}
		gdImageString(gant, gdFontSmall, iWidth - iSideBorder + 15, i * (iRowHeight + iRowGap) + iBorder + 12, s, black);
	}



}


void drawGant()
{
	/* Main function */

	int iState = 0, iOldState = -1, iFirst = TRUE;

	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	time_t t0 = 0, t1 = 0;
	#ifdef USE_DB_MYSQL
	g_string_printf(s_SQL, "SELECT ctime, txt_data, nagios_status, UNIX_TIMESTAMP(ctime) FROM perfdata_service_raw WHERE host_name = '%s'", sHost);
	#elif defined USE_DB_POSTGRESQL
	g_string_printf(s_SQL, "SELECT ctime, txt_data, nagios_status FROM perfdata_service_raw WHERE host_name = '%s'", sHost);
	#endif
	g_string_append_printf(s_SQL, " AND service_description = '%s'", sService);
	g_string_append_printf(s_SQL, " AND ctime BETWEEN '%s' AND '%s'", sFrom, sTo);
	g_string_append_printf(s_SQL, " ORDER BY perfdata_service_raw.ctime");
	query(s_SQL->str);

	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++){
	#endif

		/* Event type */
		if (sData(2)[0]) {
			iState = iData(2);
		} else {
			iState = 0;
			if (strstr(sData(1), "Connection refused"))	iState = 2;
			else if (strstr(sData(1), "OK")) 				iState = 0;
			else if (strstr(sData(1), "ok"))				iState = 0;
			else if (strstr(sData(1), "warning"))			iState = 1;
			else if (strstr(sData(1), "WARNING"))			iState = 1;
			else if (strstr(sData(1), "critical"))			iState = 2;
			else if (strstr(sData(1), "CRITICAL"))			iState = 2;
			else if (strstr(sData(1), "unknown"))			iState = 3;
			else if (strstr(sData(1), "UNKNOWN"))			iState = 3;
		}

		#ifdef USE_DB_MYSQL
		t1 = (time_t)iData(3);
		#elif defined USE_DB_POSTGRESQL
		t1 = tData(0);
		#endif
		if (t0 == 0) t0 = t1;
		if (iOldState == -1) iOldState = iState;

		if (iState == iOldState) continue;

		drawBox(t0, t1, iOldState);
		if (iFirst) {
			int getX_t0 = getX(t0);
			int getY_t0 = getY(t0);
			gdImageLine(gant, getX_t0, getY_t0, getX_t0, getY_t0 + iRowHeight, black);

			iFirst = FALSE;
		}

		t0 = t1;
		iOldState = iState;

	}

	if (t1) {
		drawBox(t0, t1, iState);
		if (iFirst)
			gdImageLine(gant, getX(t0), getY(t0), getX(t0), getY(t0) + iRowHeight, black);
		gdImageLine(gant, getX(t1), getY(t1), getX(t1), getY(t1) + iRowHeight, black);
	}

}

void drawBox(time_t t0, time_t t1, int iState)
{
	int iDay0, iDay1;
	int x0, x1, y0, y1;

	iDay0 = getDay(t0);
	iDay1 = getDay(t1);

#ifdef DEBUG
	fprintf(f, "t0 %d - %2d:%02d    t1 %d - %2d:%02d\n", 
		iDay0,
		((t0 - tFromMidnight) % DAY1) / 3600, ((t0 - tFromMidnight) % 3600) / 60,
		iDay1,
		((t1 - tFromMidnight) % DAY1) / 3600, ((t1 - tFromMidnight) % 3600) / 60);
#endif

	if (iDay0 != iDay1) {
		// Multiple day box
		time_t t;
		int iDay;
		// End of last day:
		t = t0;
		do {
			t += 3600;
			iDay = getDay(t);
		} while (iDay == iDay0);
		t -= (t % 3600) + 1;
#ifdef DEBUG
		fprintf(f, "Recall 0:\n");
#endif
		drawBox(t0, t, iState);
#ifdef DEBUG
		fprintf(f, "Recall 1:\n");
#endif
		drawBox(t + 2, t1, iState);
		return;
	}

	x0 = getX(t0);
	x1 = getX(t1);
	y0 = getY(t0);
	y1 = y0 + iRowHeight;

#ifdef DEBUG
	fprintf(f, "(%d,%d)-(%d,%d) @ %d\n", x0, y0, x1, y1, iState);
#endif

	gdImageFilledRectangle(gant, x0, y0, x1, y1, iStateColour[iState]);
	gdImageLine(gant, x0, y0, x1, y0, black);
	gdImageLine(gant, x0, y1, x1, y1, black);


}

int getX(time_t t)
{
	static double d;

	t -= tFromMidnight;
	while (t > DAY1) t -= DAY1;

	d = (double)t;

	d /= DAY1;
	d *= (iWidth - iSideBorder * 2);

	return (int)d + iSideBorder;

}

int getY(time_t t)
{
	return getDay(t) * (iRowHeight + iRowGap) + iBorder + iRowGap;
}


int getDay(time_t t)
{
	return (t - tFromMidnight) / DAY1;
}


void initChart() 
{
	int i, iR, iG, iB;

	/* Define us an area */
	gant = gdImageCreate(iWidth, iHeight);
	if (gant == NULL) exit(0);

	/* Colours (first is the background) */
	white = gdImageColorAllocate(gant, 255, 255, 255);
    black = gdImageColorAllocate(gant, 0, 0, 0);
    blue = gdImageColorAllocate(gant, 0, 0, 255);
    lblue = gdImageColorAllocate(gant, 192, 192, 255);
	red = gdImageColorAllocate(gant, 255, 0, 0);
	yellow = gdImageColorAllocate(gant, 255, 255, 0);
	grey2 = gdImageColorAllocate(gant, 192, 192, 192);
	grey1 = gdImageColorAllocate(gant, 224, 224, 224);
	grey0 = gdImageColorAllocate(gant, 240, 240, 240);

	for (i = 0; i < 4; i++) {
		sscanf(sStateColour[i], "%02X%02X%02X", &iR, &iB, &iG);
		iStateColour[i] = gdImageColorAllocate(gant, iR, iB, iG);
	}

}

void drawHeadings()
{
	int i, x;

	for (i = 0; i < 4; i++) {
		x = i * 100 + iSideBorder;
		gdImageFilledRectangle(gant, x, 4, x + 20, 13, iStateColour[i]);
		gdImageRectangle(gant, x, 4, x + 20, 13, black);
		gdImageString(gant, gdFontSmall, x + 25, 1, sStates[i], black);
	}

}



/***************************************
 *
 * Misk
 *
 ***************************************/


void getCGI()
{
	char s[20];

	getCGIDBVars();

	/* Graph details */
	sHost = strdup(scgi("host"));
	sService = strdup(scgi("service"));

	sFrom = strdup(scgi("from"));
	sTo = strdup(scgi("to"));

	tFrom = getTime(sFrom);
	tTo = getTime(sTo);
	
	strcpy(s, sFrom);
	s[10] = 0;
	sprintf(s, "%s 00:00:00", s);
	tFromMidnight = getTime(s);

	strcpy(s, sTo);
	s[10] = 0;
	sprintf(s, "%s 23:59:59", s);
	tToMidnight = getTime(s) + 1;

	iDays = (int)(tToMidnight - tFromMidnight) / DAY1;
	iHeight = iBorder * 2 + iRowHeight * iDays + iRowGap * (iDays + 1);

#ifdef DEBUG
	fprintf(f, "\"%s\" -> \"%s\" = %ld hours = %d days.\n", sFrom, sTo, 
		(tToMidnight - tFromMidnight) / 3600, iDays);
#endif

}


void print_failure_image(FILE *f, char *sMsg)
{

	gdImageFilledRectangle(gant, 0, 0, iWidth, iHeight, white);

	gdImageString(gant, gdFontSmall, 10, 10, (unsigned char*)_("Error:"), black);
	gdImageString(gant, gdFontSmall, 10, 20, sMsg, black);

	/* Splurge */
	gdImagePng(gant, stdout);

	exit(EXIT_FAILURE);

}


time_t getTime(char *sSQLDate)
/* format "YY-MM-DD hh:mm:ss" */
{
	struct tm recTime;
	static int iOffset = -1;
	time_t tOut;
	char s[5];

	if (strlen(sSQLDate) < 18) {
		int i;
		for (i = strlen(sSQLDate) + 1; i < 19; i++) sSQLDate[i] = 0;
	}

	// Year:
	s[0] = sSQLDate[0];
	s[1] = sSQLDate[1];
	s[2] = sSQLDate[2];
	s[3] = sSQLDate[3];
	s[4] = 0;
	iYear = atoi(s);
	recTime.tm_year = iYear - 1900;

	// Month:
	s[0] = sSQLDate[5];
	s[1] = sSQLDate[6];
	s[2] = 0;
	iMonth = atoi(s);
	recTime.tm_mon = iMonth - 1;

	// Month Day:
	s[0] = sSQLDate[8];
	s[1] = sSQLDate[9];
	s[2] = 0;
	iDay = atoi(s);
	recTime.tm_mday = iDay;

	// Hours:
	s[0] = sSQLDate[11];
	s[1] = sSQLDate[12];
	s[2] = 0;
	recTime.tm_hour = atoi(s);

	// Minuts:
	s[0] = sSQLDate[14];
	s[1] = sSQLDate[15];
	s[2] = 0;
	recTime.tm_min = atoi(s);

	// Seconds:
	s[0] = sSQLDate[17];
	s[1] = sSQLDate[18];
	s[2] = 0;
	recTime.tm_sec = atoi(s);
	recTime.tm_isdst = 0;

	tOut = mktime(&recTime);

	if (iOffset == -1) {
		struct tm *r;
		r = localtime(&tOut);
		r->tm_isdst = 0;
		iOffset = (int)mktime(r) - tOut;
	}

	return tOut - iOffset;
	
}



/***************************************
 *
 * dbms
 *
 ***************************************/

void doSQL()
{
	#ifdef USE_DB_MYSQL
	libpp_mysql_set_sql_failure_callback(print_sql_failure_image);
	#elif defined USE_DB_POSTGRESQL
	libpp_postgresql_set_sql_failure_callback(print_sql_failure_image);
	#endif

	now_connect(TRUE);

}

void print_sql_failure_image(char*file, int line, const char *sql, const char *error)
{
	gchar *tmpSQL, *tmpError;
	tmpSQL = g_strdup(sql);
	tmpError = g_strdup(error);

	gdImageFilledRectangle(gant, 0, 0, iWidth, iHeight, white);

	gdImageString(gant, gdFontSmall, 10, 10, (unsigned char*)_("SQL:"), black);
	gdImageString(gant, gdFontSmall, 10, 20, tmpSQL, black);

	gdImageString(gant, gdFontSmall, 10, 40, (unsigned char*)_("Error:"), black);
	gdImageString(gant, gdFontSmall, 10, 50, tmpError, black);

	g_free(tmpSQL);
	g_free(tmpError);


	/* Splurge */
	gdImagePng(gant, stdout);

	exit(EXIT_FAILURE);

}



