/*****************************************************************************
 *
 * perfparse.c - Common program to display all CGI.
 *
 * Copyright (c) 2004 Ben Clewett (bclewett@perfparse.org)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <glib.h>
#include <string.h>
#include <unistd.h>

#include <libpp_common.h>
#include "common.h"
#include "dbms.h"
#include "db_macro_actions.h"
#include "cgi.h"
#include "del_policy.h"
#include <config.h>

#include "perfgraph.h"
#include <gettext.h>
#include <locale.h>

#include "object_users.h"

/* Include of all the display sections */
#include "cgi_intro.h"
#include "cgi_graph.h"
#include "cgi_raw_report.h"
#include "cgi_bin_report.h"
#include "cgi_host_groups.h"
#include "cgi_del_policy.h"
#include "cgi_select_object.h"
#include "cgi_bin_summary_policy.h"

/* Global CGI */
int iDisplay = 0;
char sCGIOut[1024];
void getGlobalCGI();

/* Program Name */
char *sProgName;
void getProgName(char *sIn);

int main(int argc, char *argv[])
{
	char *argvInt[1];
	int   argcInt = 1;

	int options[] = {
		CONFIG_ITEM_ID_ERROR_LOG,
		CONFIG_ITEM_ID_ERROR_LOG_ROTATE,
		CONFIG_ITEM_ID_DB_USER,
		CONFIG_ITEM_ID_DB_PWD,
		CONFIG_ITEM_ID_DB_DATABASE,
		CONFIG_ITEM_ID_DB_HOST,
		CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY,
		CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_HOST_GROUP,
		CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY,
		CONFIG_ITEM_ID_DUMMY_HOSTNAME,
		CONFIG_ITEM_ID_NB
	};

	argvInt[0] = argv[0];
	
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	config_parse_config(argcInt,argvInt,"perfparse.cgi\n\n","\n",options, "stderr");

	/* Second config options, from CGI: */
	if (scgi("show_config")[0]) {
		header();
		printf("<pre>\n");
		config_show_config(stdout, "perfparse.cgi\n\n", "\n", options);
		printf("</pre>\n");
		footer();
		exit(EXIT_SUCCESS);
	}
	if (scgi("version")[0]) {
		header();
		printf("<pre>\n");
		config_print_version(stdout, "perfparse.cgi\n\n", "\n");
		printf("</pre>\n");
		footer();
		exit(EXIT_SUCCESS);
	}


	/* Get CGI variables */
	getGlobalCGI();

	/* Header */
	/* Getting more complex, need better model. */
	if (iDisplay == GRAPH_DISPLAY || iDisplay == RAW_HISTORY) {
		/* Here Be Javascript */
		headerGraphDisplay();
	} else if (iDisplay == VIEW_SAVED_GRAPH || iDisplay == ALL_RAW_DISPLAY) {
		/* No header yet. */

	} else {
		/* Default Header */
		header();
	}

	fflush(stdout);

	/* Get the program name */
	getProgName(argv[0]);

	/* Handle SQL */
	doSQL();

	/* Get the user details */
	getUser();

	/* check database version */
	if (is_db_incorrect_version()) {
		displayDatabaseError();
		footer();
		exit(1);
	}
	
	/* Diaply */
	switch (iDisplay) {
		/* Binary Summary Options */
		case DEFINE_BIN_SUMMARY:
			if (strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY), USER_PERMISSIONS_HIDE)) {
				displayDefineBinSummary();
			}
			break;

		/* Other Menus */
		case ADMIN_MENU:
			displayAdminIntro();
			break;

		case GRAPH_DISPLAY:
			displayGraphHeader();
			break;
		case ADD_AMEND_HOST_GROUP:
			if (strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_HOST_GROUP), USER_PERMISSIONS_HIDE)) 
				displayAddAmendHostGroup();
			break;
		case EDIT_HOST_GROUP:
			if (strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_HOST_GROUP), USER_PERMISSIONS_HIDE)) 
				displayEditHostGroup();
			break;
		case SELECT_HOST:
			displaySelectHost();
			break;
		case SELECT_METRIC:
			displaySelectMetric();
			break;
		case ALL_BIN_DISPLAY:
			displayAllBin();
			displayOptions();
			break;

		/* Raw Data reports */
		case RAW_HISTORY:
			displayRawHistory();
			break;
		case ALL_RAW_DISPLAY:
			displayAllRaw();
			break;
		case RAW_HISTORY_SUMMARY:
			displayRawHistorySummary();
			break;


		/* Saved Graphs */
		case SAVED_GRAPHS:
			displaySavedGraphs();
			break;
		case VIEW_SAVED_GRAPH:
			diaplaySelectedSavedGraph();
			break;
		case EDIT_SAVED_GRAPH:
			diaplayEditSavedGraph();
			break;

		case DELETE_POLICY_BIN_DISPLAY:
			if(strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY), USER_PERMISSIONS_HIDE)) {
				validateDefaultDP();
				check_db_integrity();
				displayDeletePolicyBin();
			}
			break;
		case DELETE_POLICY_RAW_DISPLAY:
			if(strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY), USER_PERMISSIONS_HIDE)) {
				validateDefaultDP();
				check_db_integrity();
				displayDeletePolicyRaw();
			}
			break;
		case EDIT_DELETE_POLICIES:
			if(strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY), USER_PERMISSIONS_HIDE)) {
				validateDefaultDP();
				check_db_integrity();
				displayEditDeletePolicies();
			}
			break;
		case EDIT_HOST_DELETE_POLICIES:
			if(strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY), USER_PERMISSIONS_HIDE)) {
				validateDefaultDP();
				check_db_integrity();
				displayEditHostDefaults();
			}
			break;
		case MARK_FOR_DELETION:
			markForDeletion();
			break;
		default:
			displayIntro();
			break;
	}

	/* footer */
	footer();

	#ifdef USE_DB_MYSQL
	mysql_close(&db_mysql);
	#elif defined USE_DB_POSTGRESQL
	PQfinish(db_postgresql);
	#endif
	
	exit(0);

}



/***************************************
 *
 * Displays
 *
 ***************************************/



void displayDatabaseError()
{


	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");

	printf("  <tr>\n");
	printf("    <td width=60><img src=\"%s%s/perfparse-logo-sm.png\" border=0></td>\n", 
		(HTTP_IMAGE_PATH[0] == '/') ? "" : "/", HTTP_IMAGE_PATH);
	printf("    <td>%s\n", FONT);
	printf("      <p align=center><b>%s</b></p>\n",_("Database Schema version is incorrect."));
	setlocale(LC_NUMERIC, "POSIX");
	printf(_("&nbsp;&nbsp;Current Version: %.2lf, Required Version: %.2f."),
		getVersion(), DB_VERSION);
	setlocale(LC_NUMERIC, "");
	printf("<br>\n");
	printf("&nbsp;&nbsp;%s<br>\n",_("Consider upgrading with:"));
	printf("&nbsp;&nbsp;<font face=\"Courier\">perfparse-db-tool --update</font>\n");
	printf("    </font></td>\n");
	printf("    <td width=60 align=right>%sv%s</td>\n", FONT_SMALL, VERSION);
	printf("  </tr>\n");
	printf("  </table>\n");

	printf("</td></tr></table></p>\n");

}


void displayUnsupported()
{

	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");

	printf("  <tr>\n");
	printf("    <td width=60><img src=\"%s%s/perfparse-logo-sm.png\" border=0></td>\n", 
		(HTTP_IMAGE_PATH[0] == '/') ? "" : "/", HTTP_IMAGE_PATH);
	printf("    <td align=center>%s\n", FONT);
	printf("      <b>%s</b><br>\n",_("This option is currently unsupported."));
	printf("      %s<br>\n",_("Please check the <a href=\"http://www.perfparse.org\">project</a> page for more details."));
	printf("      <a href=\"%s\">%s</a>\n", sProgName,_("Return to main menu"));
	printf("    </font></td>\n");
	printf("    <td width=60 align=right>%sv%s</td>\n", FONT_SMALL, VERSION);
	printf("  </tr>\n");
	printf("  </table>\n");

	printf("</td></tr></table></p>\n");

}







void displayOptions()
{
	/* Database option no longer requested.  But variables still respected if passed */

	return;

	/*
	printf("<p align=center><table width=400 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");

	printf("<form method=get>\n");
	printf("<table width=\"100%%\" border=0 cellpadding=0 cellspacing=5>\n");
	printf("<tr>\n");
	printf("  <td colspan=4 align=center><font face=\"Arial, Helvetica\" size=2><b>Database Details</b></td>\n");
	printf("</tr>\n");

	printf("<tr><td colspan=4><hr width=\"100%%\"></td></tr>\n");
	
	printf("<tr>");
	printf("  <td align=right><font face=\"Arial, Helvetica\" size=2>Server:</td>\n");
	printf("  <td><input type=text name=\"db_host\" value=\"%s\" size=20 maxlength=70></td>\n", sDBHost);
	printf("  <td align=right><font face=\"Arial, Helvetica\" size=2>Database:</td>\n");
	printf("  <td><input type=text name=\"db_name\" value=\"%s\" size=20 maxlength=70></td>\n", sDBName);
	printf("</tr><tr>\n");
	printf("  <td align=right><font face=\"Arial, Helvetica\" size=2>Username:</td>\n");
	printf("  <td><input type=text name=\"db_user\" value=\"%s\" size=20 maxlength=70></td>\n", sDBUser);
	printf("  <td align=right><font face=\"Arial, Helvetica\" size=2>Password:</td>\n");
	printf("  <td><input type=text name=\"db_pass\" value=\"%s\" size=20 maxlength=70></td>\n", sDBPass);
	printf("</tr>\n");

	printf("<tr><td colspan=4><hr width=\"100%%\"></td></tr>\n");

	printf("<tr>\n");
	printf("  <td colspan=4 align=center><input type=submit value=\"Enter\"</td>\n");
	printf("</tr>\n");
	printf("</table>\n");

	printf("</td></form></tr></table></p>\n");
	*/

}


void footer()
{
	printf("</body></html>");
}

void header_meta()
{
	printf("Expires: Sun, 02 Jan 2000 01:01:01 GMT\n");
   	printf("content-type: text/html\n\n");
}

void header()
{
	header_meta();

	printf("<HTML>\n");
   	printf("<HEAD>\n");
   	printf("<TITLE>\n");
    printf("PerfParse");    
	printf("</TITLE>\n");
    printf("<BODY BGcolor=\"#EEFFFF\" TEXT=\"#000000\" LINK=\"#000000\" VLINK=\"#000000\" ALINK=\"#000000\">\n"); 
}

void header_refresh(int iSeconds)
{
	printf("<HTML>\n");
   	printf("<HEAD>\n");
   	printf("<TITLE>\n");
    printf("PerfParse");    
	printf("</TITLE>\n");
	if (iSeconds) {
		printf("<meta http-equiv=\"REFRESH\" content=\"%d\"></HEAD>\n", iSeconds);
	}
    printf("<BODY BGcolor=\"#EEFFFF\" TEXT=\"#000000\" LINK=\"#000000\" VLINK=\"#000000\" ALINK=\"#000000\">\n"); 
}

void headerGraphDisplay()
{
	header_meta();

	printf("<HTML>\n");
   	printf("<HEAD>\n");
   	printf("<TITLE>\n");
    printf("PerfParse");    
	printf("</TITLE>\n");

	// Javascript to show/hide controls:
	printf("<SCRIPT LANGUAGE=\"JavaScript\" TYPE=\"text/javascript\">\n");
	printf("function isAbsRelVisible() {\n");
	printf("  if (document.graph.timerange[1].checked) showAbsolute();\n");
	printf("  else showRelative();\n");
	printf("}\n");
	printf("function showAbsolute() {\n");
	printf("   Abs1.style.visibility=\"visible\";\n");
	printf("   Abs2.style.visibility=\"visible\";\n");
	printf("   Abs3.style.visibility=\"visible\";\n");
	printf("   Abs4.style.visibility=\"visible\";\n");
	printf("   Rel1.style.visibility=\"hidden\";\n");
	printf("   Rel2.style.visibility=\"hidden\";\n");
	printf("   Rel3.style.visibility=\"hidden\";\n");
	printf("   Rel4.style.visibility=\"hidden\";\n");
	printf("}\n");
	printf("function showRelative() {\n");
	printf("   Abs1.style.visibility=\"hidden\";\n");
	printf("   Abs2.style.visibility=\"hidden\";\n");
	printf("   Abs3.style.visibility=\"hidden\";\n");
	printf("   Abs4.style.visibility=\"hidden\";\n");
	printf("   Rel1.style.visibility=\"visible\";\n");
	printf("   Rel2.style.visibility=\"visible\";\n");
	printf("   Rel3.style.visibility=\"visible\";\n");
	printf("   Rel4.style.visibility=\"visible\";\n");
	printf("}\n");
	printf("</SCRIPT>\n");

    printf("<BODY");
	printf(" BGcolor=\"#EEFFFF\" TEXT=\"#000000\" LINK=\"#000000\" VLINK=\"#000000\" ALINK=\"#000000\"");
	printf(" onload=\"isAbsRelVisible()\"");
	printf(">\n"); 
}



/*
void doXML()
{
	// No graph, just show numbers as a XML
	MYSQL_RES	*result;

	printf("Expires: Sun, 02 Jan 2000 01:01:01 GMT\n");
   	printf("content-type: xml\n\n");

	printf("<xml>\n");
	printf("<meta http-equiv=\"Content-Type\" content=\"XML; charset=utf-8\"/>\n");
	printf("<meta name=\"Generator\" content=\"Nagios Performance Data\"/>\n");
	printf("<perfparse host=\"%s\"", sql_escape(sHost,PERCENT_ESCAPE_CHAR));
	printf(" service=\"%s\"", sql_escape(sService,PERCENT_ESCAPE_CHAR));
	printf(" metric=\"%s\" from=\"%s\" to=\"%s\"/>\n", sql_escape(sMetric,PERCENT_ESCAPE_CHAR), sFrom, sTo);


	g_string_printf(s_SQL, "SELECT ctime, value, warn, critical FROM perfdata_service_bin WHERE host_name = '%s'",
		sql_escape(sHost,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND service_description = '%s'",
		sql_escape(sService,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND metric = '%s'",
		sql_escape(sMetric,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND ctime BETWEEN '%s' AND '%s'",
		sFrom, sTo);

	if (mysql_query(&db_mysql, s_SQL->str)) sql_fail(s_SQL->str);
	result = mysql_use_result(&db_mysql);

	while ((result_row = mysql_fetch_row(result))) {

		printf("<row time=\"%s\" val=\"%s\" warn=\"%s\" crit=\"%s\"/>\n",
			result_row[0], result_row[1], result_row[2], result_row[3]);

	}

	if (result) mysql_free_result(result);

	printf("</xml>\n");

}
*/

/*
void doCSV()
{
	// No graph, just show numbers as a CSV 
	MYSQL_RES	*result;

	printf("Expires: Sun, 02 Jan 2000 01:01:01 GMT\n");
   	printf("content-type: CSV\n\n");


	g_string_printf(s_SQL, "SELECT ctime, value, warn, critical FROM perfdata_service_bin WHERE host_name = '%s'",
		sql_escape(sHost,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND service_description = '%s'",
		sql_escape(sService,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND metric = '%s'",
		sql_escape(sMetric,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND ctime BETWEEN '%s' AND '%s'",
		sFrom, sTo);

	printf("Time,value,Warn,Critial\n");

	if (mysql_query(&db_mysql, s_SQL->str)) sql_fail(s_SQL->str);
	result = mysql_use_result(&db_mysql);

	while ((result_row = mysql_fetch_row(result))) {

		printf("%s,%s,%s,%s\n",
			result_row[0], result_row[1], result_row[2], result_row[3]);

	}

	if (result) mysql_free_result(result);

}
*/

/***************************************
 *
 * Misk
 *
 ***************************************/

char *getPath(char *sIn)
{

	int i, iSlash = 0;

	if (!sIn) return "";

	for (i = 0; sIn[i]; i++) {
		if (sIn[i] == '/') iSlash = i;
	}

	if (iSlash) sIn[iSlash] = 0;

	return sIn;
}



void getGlobalCGI()
{

	getCGIDBVars();

	/* What to display */
	if (icgi("graph", 0)) 
		iDisplay = GRAPH_DISPLAY;
	else if (icgi("all_bin", 0)) 
		iDisplay = ALL_BIN_DISPLAY;
	else if (icgi("delete_policy_bin", 0)) 
		iDisplay = DELETE_POLICY_BIN_DISPLAY;
	else if (icgi("delete_policy_raw", 0)) 
		iDisplay = DELETE_POLICY_RAW_DISPLAY;
	else if (icgi("add_amend_host_group", 0))
		iDisplay = ADD_AMEND_HOST_GROUP;
	else if (icgi("edit_host_list", 0))
		iDisplay = EDIT_HOST_GROUP;
	else if (icgi("select_host", 0))
		iDisplay = SELECT_HOST;
	else if (icgi("select_metric", 0))
		iDisplay = SELECT_METRIC;
	else if (icgi("edit_delete_policies", 0))
		iDisplay = EDIT_DELETE_POLICIES;
	else if (icgi("edit_host_delete_policies", 0))
		iDisplay = EDIT_HOST_DELETE_POLICIES;
	else if (icgi("saved_graphs", 0))
		iDisplay = SAVED_GRAPHS;
	else if (icgi("mark_for_deletion", 0))
		iDisplay = MARK_FOR_DELETION;
	else if (icgi("view_saved_graph", 0))
		iDisplay = VIEW_SAVED_GRAPH;
	else if (icgi("edit_saved_graph", 0))
		iDisplay = EDIT_SAVED_GRAPH;

	/* Raw data reports */
	else if (icgi("raw_history_summary", 0))
		iDisplay = RAW_HISTORY_SUMMARY;
	else if (icgi("raw_history", 0))
		iDisplay = RAW_HISTORY;
	else if (icgi("all_raw", 0)) 
		iDisplay = ALL_RAW_DISPLAY;

	/* Binary Summary Options */
	else if (icgi("define_bin_summary", 0))
		iDisplay = DEFINE_BIN_SUMMARY;

	/* Menus */
	else if (icgi("admin_menu", 0))
		iDisplay = ADMIN_MENU;

}

/*
void fail(FILE *f, char *sMsg)
{

	printf("<pre>\n");
	printf("Error:\n%s\n", sMsg);
	printf("</pre>\n");

	footer();

	exit(-1);

}
*/


/***************************************
 *
 * dbms
 *
 ***************************************/

void addHostToGroup(char *sHostName, char *sGroupName) 
{
	g_string_printf(s_SQL, "UPDATE perfdata_host SET group_name = '%s'", sql_escape(sGroupName,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " WHERE host_name = '%s'", sql_escape(sHostName,PERCENT_ESCAPE_CHAR));

	query_no_return(s_SQL->str);
}


void deleteHostFromGroup(char *sHostName)
{
	g_string_printf(s_SQL, "UPDATE perfdata_host SET group_name = NULL where host_name = '%s'",
			sql_escape(sHostName,PERCENT_ESCAPE_CHAR));

	query_no_return(s_SQL->str);
}

void addGroup(char *sGroupName)
{
	g_string_printf(s_SQL, "SELECT group_name FROM perfdata_host_group WHERE group_name = '%s'", sql_escape(sGroupName,PERCENT_ESCAPE_CHAR));

	query(s_SQL->str);

	if (rows()) return;

	g_string_printf(s_SQL, "INSERT INTO perfdata_host_group (group_name) VALUES ('%s')", sql_escape(sGroupName,PERCENT_ESCAPE_CHAR));

	query_no_return(s_SQL->str);

}

void deleteGroup(char *sGroupName)
{
	g_string_printf(s_SQL, "DELETE FROM perfdata_host_group WHERE group_name = '%s'", sql_escape(sGroupName,PERCENT_ESCAPE_CHAR));

	query_no_return(s_SQL->str);

}

void print_sql_failure_html(char*file,int num, const char *sql, const char *error)
{
	printf("<pre>");
	printf(_("An error occured with the SQL:\n\n  \"%s\"\n\nFailure Message:\n\n  \"%s\""),
		sql, error);
	printf("</pre>");

	footer();

	exit(EXIT_FAILURE);
}



void doSQL()
{
	#ifdef USE_DB_MYSQL
	libpp_mysql_set_sql_failure_callback(print_sql_failure_html);
	#elif defined USE_DB_POSTGRESQL
	libpp_postgresql_set_sql_failure_callback(print_sql_failure_html);
	#endif

	now_connect(TRUE);

}





void getTimeRange(time_t tIn, char *sFrom, char *sTo)
{
	/* Get a relative time into an absolute time */

	time_t t0, t1;
	struct tm *lt;

	t1 = time(NULL);
	t0 = t1 - tIn;

	lt = localtime(&t0);
	sprintf(sFrom, "%04d-%02d-%02d %02d:%02d:%02d",
		lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min, lt->tm_sec);

	lt = localtime(&t1);
	sprintf(sTo, "%04d-%02d-%02d %02d:%02d:%02d",
		lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min, lt->tm_sec);

}





char *getDecInc(char *sDefault)
{
        static char *sIn;
        static char sOut[20];

        sIn = scgi("inc");

        if (sIn[0]) {
                sprintf(sOut, "ORDER BY %s", sIn);
                return sOut;
        }

        sIn = scgi("dec");

        if (sIn[0]) {
                sprintf(sOut, "ORDER BY %s DESC", sIn);
                return sOut;
        }

        return sDefault;
}

void displayDecInc(char *sField, char *sURL, char *sDefault)
{
        static char *sIn;
        static int  iInc, iDec;
        static int bHit;

        bHit = FALSE;

        sIn = scgi("inc");
        if (sIn[0]) bHit = TRUE;
        if (!strcmp(sField, sIn)) iInc = 1; else iInc = 0;

        sIn = scgi("dec");
        if (sIn[0]) bHit = TRUE;
        if (!strcmp(sField, sIn)) iDec = 1; else iDec = 0;

        if (!bHit) {
                if (!strcmp(sField, sDefault)) iInc = 1;
        }

        printf("<a href=\"%s&inc=%s\"><img src=\"%s%s/inc%d.png\" border=0></a>",
                sURL, url_encode(sField),
                (HTTP_IMAGE_PATH[0] == '/') ? "" : "/", HTTP_IMAGE_PATH,
                iInc);


        printf("<a href=\"%s&dec=%s\"><img src=\"%s%s/dec%d.png\" border=0></a>",
                sURL, url_encode(sField),
                (HTTP_IMAGE_PATH[0] == '/') ? "" : "/", HTTP_IMAGE_PATH,
                iDec);

        printf("&nbsp;");

}




void getProgName(char *sIn)
{
	int i;

	sProgName = sIn;

	for (i = 0; sIn[i]; i++) if (sIn[i] == '/' && sIn[i+1]) {
		sProgName = sIn + i + 1;
		i++;
	}


}


void getRelativeToAbsolte(int iDays, char *sTime, time_t *t0, time_t *t1, char *sFromIn, char *sToIn)
// Get a relative time into an absolute time 
{
	struct tm *lt;
	char sHour[4] = "", sMinute[4] = "";

	iDays *= 3600 * 24;
	sscanf(sTime, "%[^:]:%s", sHour, sMinute);
	iDays += atoi(sHour) * 3600 + atoi(sMinute) * 60;

	*t1 = time(NULL);
	*t0 = *t1 - (time_t)iDays;

	lt = localtime(t0);
	sprintf(sFromIn, "%04d-%02d-%02d %02d:%02d:%02d",
		lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min, lt->tm_sec);

	lt = localtime(t1);
	sprintf(sToIn, "%04d-%02d-%02d %02d:%02d:%02d",
		lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min, lt->tm_sec);

}


/**********************************************************************
 *
 *  Misk CGI bits
 *
 **********************************************************************/


void showHeader(char *sTitle, char *sExtraOptions)
{

	printf("<p align=center>\n");

	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");

	printf("  <tr>\n");

	printf("    <td width=60><img src=\"%s%s/perfparse-logo-sm.png\" border=0></td>\n", 
		(HTTP_IMAGE_PATH[0] == '/') ? "" : "/", HTTP_IMAGE_PATH);

	printf("    <td align=center>%s\n", FONT);
	printf("      <b>%s</b>\n", sTitle);
	printf("      <br>\n");
	if (sExtraOptions && sExtraOptions[0]) {
		printf("      %s\n", sExtraOptions);
		printf("      <br>\n");
	}
	printf("      <a href=\"%s\">%s</a>\n", sProgName,_("Return to main menu"));
	printf("    </font></td>\n");

	printf("    <td width=60 align=right>%sv%s</font></td>\n", FONT_SMALL, VERSION);

	printf("  </tr>\n");
	printf("</table>\n");

	printf("</td></tr></table>\n");
	
	printf("</p>\n");

}


void selectGroup(char *sMenuOptionName, int bTerminateTable) 
{
	char *sGroupNameCGI;
	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	sGroupNameCGI = scgi("group_name");

	if (sGroupNameCGI[0]) {
		free(user.sGroupName);
		user.sGroupName = strdup(sGroupNameCGI);
		saveUser();
	}

	printf("<form method=\"get\">\n");
	printf("<input type=hidden name=\"%s\" value=\"1\">\n", sMenuOptionName);

	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");

	printf("  <tr>\n");
	printf("    <td align=center width=\"33%%\"><font face=\"Arial, Helvetica\" size=2>\n");
	printf("      %s\n",_("Select Host Group:"));
	printf("    </td>\n");
	printf("    <td>\n");
	printf("       <select name=\"group_name\">\n");
	printf("<option value=\"\">%s\n",_("-- Select group: --"));
	printf("<option value=\"**ALL**\"%s>&lt;%s&gt;\n", (!strcmp("**ALL**", user.sGroupName)) ? " selected" : "",_("All Groups"));

	query("SELECT group_name FROM perfdata_host_group");
	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
		printf("<option%s>%s\n", (!strcmp(sData(0), user.sGroupName)) ? " selected" : "", sData(0));
	}
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++){
		printf("<option%s>%s\n", (!strcmp(sData(0), user.sGroupName)) ? " selected" : "", sData(0));
	}
	#endif
	
	printf("       </select>\n");
	printf("       <input type=submit value=\"%s\">\n",_("Enter"));
	printf("    </td></form>\n");
	printf("  </tr>\n");

	if (bTerminateTable) {
		printf("  </table>\n");
		printf("</td></tr></table></p>\n");
	}

}




