/*****************************************************************************
 *
 * del_policy.c - Common functions required by PerfParse suit.
 *
 * Copyright (c) 2004 Ben Clewett (perfparse@clewett.org.uk)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <glib.h>
#include <string.h>
#include <unistd.h>

#include "libpp_common.h"
#include "common.h"
#include "dbms.h"
#include "del_policy.h"
#include <config.h>
#include <gettext.h>

char sError[100];

/* ALWAYS in order 'bin', 'raw' */
const char *sDefaultDP[] = { STRING_DELETE_POLICY_BIN_DEFAULT, STRING_DELETE_POLICY_RAW_DEFAULT, "" };


/*************************************************** 
 *
 * Working with perfdata_delete_policy records 
 *
 ***************************************************/


void editPolicy(char *sPolicyName, int iPolicy) 
{

	if (isPolicyDefined(sPolicyName)) {

		g_string_printf(s_SQL, "UPDATE perfdata_delete_policy SET policy_name = '%s'",
			sql_escape(sPolicyName,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, ", delete_policy = %s WHERE policy_name = '%s'",
			pval2sql(iPolicy), sql_escape(sPolicyName,PERCENT_ESCAPE_CHAR));

	} else {

		g_string_printf(s_SQL, "INSERT perfdata_delete_policy (policy_name, delete_policy) VALUES ('%s',%s)",
			sql_escape(sPolicyName,PERCENT_ESCAPE_CHAR), pval2sql(iPolicy));
	}

	query_no_return(s_SQL->str);

}



void deletePolicyName(char *sPolicy) 
{

	g_string_printf(s_SQL, "DELETE FROM perfdata_delete_policy WHERE policy_name = '%s'", sql_escape(sPolicy,PERCENT_ESCAPE_CHAR));

	query_no_return(s_SQL->str);


}

int isPolicyDefined(char *sPolicyName)
{

	g_string_printf(s_SQL, 
		"SELECT COUNT(*) FROM perfdata_delete_policy WHERE policy_name = '%s'", 
		sql_escape(sPolicyName,PERCENT_ESCAPE_CHAR));

	return (query_int(s_SQL->str)) ? TRUE : FALSE;

	
}

int getPolicy(char *sPolicyName)
{

	g_string_printf(s_SQL, 
		"SELECT delete_policy FROM perfdata_delete_policy WHERE policy_name = '%s'", 
		sql_escape(sPolicyName,PERCENT_ESCAPE_CHAR));

	return sql2pval(query_char(s_SQL->str));

}


/*************************************************** 
 *
 * Working with delete_policy record on tables.
 *
 ***************************************************/


void setRawPolicy(char *sHostIn, char *sServiceIn, int iPolicy, int iPolicyType, char *sTemplate)
{
	char sName[31];

	if (sTemplate && sTemplate[0]) sprintf(sName, "'%s'", sql_escape(sTemplate,PERCENT_ESCAPE_CHAR));
	else strcpy(sName, "NULL");

	g_string_printf(s_SQL, "UPDATE perfdata_service SET raw_delete_policy = %s, raw_delete_policy_type = %s",
		pval2sql(iPolicy), ptype2sql(iPolicyType));
	g_string_append_printf(s_SQL, ", raw_delete_policy_name = %s", sName);
	g_string_append_printf(s_SQL, " WHERE host_name = '%s'",
		sql_escape(sHostIn,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND service_description = '%s'",
		sql_escape(sServiceIn,PERCENT_ESCAPE_CHAR));
	query_no_return(s_SQL->str);

}


void setBinPolicy(char *sHostIn, char *sServiceIn, char *sMetricIn, int iPolicy, int iPolicyType, char *sTemplate)
{

	char sName[31];

	if (sTemplate && sTemplate[0]) sprintf(sName, "'%s'", sql_escape(sTemplate,PERCENT_ESCAPE_CHAR));
	else strcpy(sName, "NULL");

	g_string_printf(s_SQL, "UPDATE perfdata_service_metric SET bin_delete_policy = %s, bin_delete_policy_type = %s",
		pval2sql(iPolicy), ptype2sql(iPolicyType)); 
	g_string_append_printf(s_SQL, ", bin_delete_policy_name = %s", sName);
	g_string_append_printf(s_SQL, " WHERE host_name = '%s'",
		sql_escape(sHostIn,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND service_description = '%s'",
		sql_escape(sServiceIn,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " AND metric = '%s'",
		sql_escape(sMetricIn,PERCENT_ESCAPE_CHAR));
	query_no_return(s_SQL->str);

}

void setBinHostPolicyUser(char *sHostIn, int iPolicy)
{

	g_string_printf(s_SQL, "UPDATE perfdata_host SET bin_delete_policy = %s, bin_delete_policy_type = 'user' WHERE host_name = '%s'",
		pval2sql(iPolicy), sql_escape(sHostIn,PERCENT_ESCAPE_CHAR));
	
	query_no_return(s_SQL->str);

}


void setBinHostPolicyTemplate(char *sHostIn, char *sTemplate)
{

	g_string_printf(s_SQL, "UPDATE perfdata_host SET bin_delete_policy = %s, bin_delete_policy_type = 'template'",
		pval2sql(getPolicy(sTemplate)));
	g_string_append_printf(s_SQL, ", bin_delete_policy_name = '%s'",
		sql_escape(sTemplate,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, "	WHERE host_name = '%s'",
		sql_escape(sHostIn,PERCENT_ESCAPE_CHAR));
	query_no_return(s_SQL->str);

}


void setRawHostPolicyUser(char *sHostIn, int iPolicy)
{

	g_string_printf(s_SQL, "UPDATE perfdata_host SET raw_delete_policy = %s, raw_delete_policy_type = 'user' WHERE host_name = '%s'",
		pval2sql(iPolicy), sql_escape(sHostIn,PERCENT_ESCAPE_CHAR));
	query_no_return(s_SQL->str);

}


void setRawHostPolicyTemplate(char *sHostIn, char *sTemplate)
{

	g_string_printf(s_SQL, "UPDATE perfdata_host SET raw_delete_policy_type = 'template', raw_delete_policy = %s",
		pval2sql(getPolicy(sTemplate)));
	g_string_append_printf(s_SQL, ", raw_delete_policy_name = '%s'",
		sql_escape(sTemplate,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, "	WHERE host_name = '%s'",
		sql_escape(sHostIn,PERCENT_ESCAPE_CHAR));

	query_no_return(s_SQL->str);

}




/*************************************************** 
 *
 * General Helpers
 *
 ***************************************************/

char *pval2sql(int iPolicy) 
{
	static char sPolicy[10];

	if (iPolicy < 0)
		strcpy(sPolicy, "NULL");
	else
		sprintf(sPolicy, "%d", iPolicy);

	return sPolicy;
}

int sql2pval(const char *sPolicy)
{

	if (!sPolicy || !strcmp(sPolicy, "NULL") || sPolicy[0] == 0) return -1;

	return atoi(sPolicy);

}

char *ptype2sql(int iType)
{
	switch (iType) {
		case DP_USER:		return "'user'";
		case DP_HOST:		return "'host'";
		case DP_TEMPLATE:	return "'template'";
	}

		/* FIXME: used to be a call to fail() */
		pp_log(__FILE__,__LINE__, "Unknown template type '%d' requested.", iType);
		exit(EXIT_FAILURE);

}

int sql2ptype(const char *sIn)
{
	if (!strcmp(sIn, "user")) return DP_USER;
	if (!strcmp(sIn, "host")) return DP_HOST;
	if (!strcmp(sIn, "template")) return DP_TEMPLATE;

		/* FIXME: used to be a call to fail() */
		pp_log(__FILE__,__LINE__, "Unknown template type '%s' requested.", sIn);
		exit(EXIT_FAILURE);
}


void validateDefaultDP()
{
	/* Ensure the default policies exist. */

	static int bChecked = FALSE;
	int i = 0;
	const char *s;

	if (bChecked) return;

	for (s = sDefaultDP[0]; sDefaultDP[i][0]; s = sDefaultDP[++i]) {

		g_string_printf(s_SQL, "SELECT count(*) FROM perfdata_delete_policy WHERE policy_name = '%s'",
			sql_escape(gettext(s),PERCENT_ESCAPE_CHAR));

		if (!query_int(s_SQL->str)) {

			g_string_printf(s_SQL, "INSERT perfdata_delete_policy (policy_name, delete_policy) VALUES ('%s',%d)",
				sql_escape(gettext(s),PERCENT_ESCAPE_CHAR), DEFAULT_DELETE_POLICY_DAYS);
			query_no_return(s_SQL->str);

		}

	}

	bChecked = TRUE;

}

int  isDefaultDP(char *sPolicy)
{
	/* Is this a dafault policy */

	int i = 0;
	const char *s;

	for (s = sDefaultDP[0]; sDefaultDP[i][0]; s = sDefaultDP[++i])
		if (!strcmp(sPolicy, gettext(s))) return TRUE;

	return FALSE;

}


int getDefaultPolicy(int i)
{
	g_string_printf(s_SQL, "SELECT delete_policy FROM perfdata_delete_policy WHERE policy_name = '%s'",
		sql_escape(gettext(sDefaultDP[i]),PERCENT_ESCAPE_CHAR));

	return sql2pval(query_char(s_SQL->str));

}

int getDefaultBinPolicy()
{
	return getDefaultPolicy(0);
}

int getDefaultRawPolicy()
{
	return getDefaultPolicy(1);
}

char *getPolicyDesc(int iPolicy)
{
	static char s[20];

	switch (iPolicy) {
		case 0:
			strcpy(s, "Always");
			break;
		case -1:
			strcpy(s, "Never");
			break;
		default:
			sprintf(s, "%d Day%s", iPolicy, (iPolicy == 1) ? "" : "s");
			break;
	}

	return s;

}


int  getRawHostPolicy(char *sHost) 
{
	g_string_printf(s_SQL, "SELECT raw_delete_policy FROM perfdata_host WHERE host_name = '%s'",
		sql_escape(sHost,PERCENT_ESCAPE_CHAR));

	return sql2pval(query_char(s_SQL->str));
}

int  getBinHostPolicy(char *sHost) 
{
	g_string_printf(s_SQL, "SELECT bin_delete_policy FROM perfdata_host WHERE host_name = '%s'",
		sql_escape(sHost,PERCENT_ESCAPE_CHAR));

	return sql2pval(query_char(s_SQL->str));
}


TPolicyList *getPolicyList()
{
	static TPolicyList *first = NULL;
	TPolicyList *p, *prev;

	if (first) destroyPolicyList(first);

	first = NULL;
	prev = NULL;

	query("SELECT delete_policy, policy_name FROM perfdata_delete_policy ORDER BY policy_name");

	while ((result_row = mysql_fetch_row(query_result))) {

		p = (TPolicyList *)malloc(sizeof(TPolicyList));
		p->iPolicy = sql2pval(sData(0));
		p->sName = strdup(sData(1));
		p->next = NULL;

		if (!first) {
			first = p;
			prev = p;
		} else {
			prev->next = p;
			prev = p;
		}

	}

	return first;
}


void destroyPolicyList(TPolicyList *first)
{
	/* [TODO] */
}



