/*****************************************************************************
 *
 * dbms.c - Common functions required by PerfParse suit.
 *
 * Copyright (c) Fabalabs Software GmbH, 2005
 *   oss@fabalabs.org
 *   Author: Friedrich Priewasser (friedrich.priewasser@fabalabs.org)
 *   Last Modified:   2005-01-13
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <glib.h>
#include "libpp_common.h"
#include "common.h"
#include <netdb.h>
#include <dbms.h>
#include <config.h>
#include <gettext.h>

#undef SHOW_SQL
//#define SHOW_SQL

sql_failure_t sql_failure = NULL;
void freeResult(void);

/***************************************
 *
 * DBMS work
 *
 ***************************************/

/* DBMS identity: */
// char*			db_port = DEF_PGPORT_STR;
char*			db_port = "5432";
PGconn		*db_postgresql;
PGresult		*query_result = NULL;
int			currentRow;

/* State Variables */
GString *s_SQL=NULL;
int 	iSQLCount = 0;
int	iRowsAffected;

int iData(int iCol)
{	
	char* result=PQgetvalue(query_result,currentRow,iCol);
	if (result && result[0] ) 
		return atoi(result );
	else
		return 0;
}

int isNull(int iCol)
{
	if (PQgetisnull(query_result,currentRow,iCol)) 
		return TRUE;
	return FALSE;
}

char *sData(int iCol)
{
	char* result=PQgetvalue(query_result,currentRow,iCol);
	if (result && result[0]) 
		return result;
	else
		return "";
}

int bData(int iCol)
{
	char* result=PQgetvalue(query_result,currentRow,iCol);
	if (result && result[0])
	{ 
		if (result[0] == 't' || result[0] == 'T' || result[0] == '1') 
			return 1;
		else
			return 0;
	}
	else
	{
		return 0;
	}
}

time_t tData(int iCol)
{
	char* result=PQgetvalue(query_result,currentRow,iCol);
	if (result && result[0])
		return getTimeFromString(result);
	else
		return (time_t)0;
}

time_t getTimeFromString(char *sYYYYMMDDhhmmss)
// Format 'YYYY-MM-DD hh:mm:ss' -> time_t
{
	struct tm recTime;
	static int iOffset = -1;
	time_t tOut;
	char s[5];

	if (strlen(sYYYYMMDDhhmmss) < 18) {
		int i;
		for (i = strlen(sYYYYMMDDhhmmss) + 1; i < 19; i++) sYYYYMMDDhhmmss[i] = 0;
	}

	// Year:
	s[0] = sYYYYMMDDhhmmss[0];
	s[1] = sYYYYMMDDhhmmss[1];
	s[2] = sYYYYMMDDhhmmss[2];
	s[3] = sYYYYMMDDhhmmss[3];
	s[4] = 0;
	recTime.tm_year = atoi(s) - 1900;

	// Month:
	s[0] = sYYYYMMDDhhmmss[5];
	s[1] = sYYYYMMDDhhmmss[6];
	s[2] = 0;
	recTime.tm_mon = atoi(s) - 1;

	// Month Day:
	s[0] = sYYYYMMDDhhmmss[8];
	s[1] = sYYYYMMDDhhmmss[9];
	s[2] = 0;
	recTime.tm_mday = atoi(s);

	// Hours:
	s[0] = sYYYYMMDDhhmmss[11];
	s[1] = sYYYYMMDDhhmmss[12];
	s[2] = 0;
	recTime.tm_hour = atoi(s);

	// Minuts:
	s[0] = sYYYYMMDDhhmmss[14];
	s[1] = sYYYYMMDDhhmmss[15];
	s[2] = 0;
	recTime.tm_min = atoi(s);

	// Seconds:
	s[0] = sYYYYMMDDhhmmss[17];
	s[1] = sYYYYMMDDhhmmss[18];
	s[2] = 0;
	recTime.tm_sec = atoi(s);
	recTime.tm_isdst = 0;

	tOut = mktime(&recTime);

	if (iOffset == -1) {
		struct tm *r;
		r = localtime(&tOut);
		r->tm_isdst = 0;
		iOffset = (int)mktime(r) - tOut;
	}

	return tOut - iOffset;
	
}

void writeDate(time_t t, char *s)
{
	struct tm *lt;

	lt = localtime(&t);
	sprintf(s, "%04d-%02d-%02d %02d:%02d:%02d",
		lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min, lt->tm_sec);

}

void pp_postgresql_default_sql_failure(char*f,int l, const char*str, const char *error) {
	pp_log(f,l,"%s (%s)",str,error);
	exit(EXIT_FAILURE);
}

void libpp_postgresql_set_sql_failure_callback(sql_failure_t f) {
	sql_failure = f;
}

void libpp_postgresql_query(char*file, int line, const char *sql)
{
	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	#endif
	freeResult();

	query_result = PQexec(db_postgresql, sql);
	if(PQresultStatus(query_result) == PGRES_BAD_RESPONSE || PQresultStatus(query_result) == PGRES_FATAL_ERROR)
		sql_failure(file, line, sql, PQresultErrorMessage(query_result));

	iRowsAffected = atoi(PQcmdTuples(query_result));
	
	#ifdef SHOW_SQL
	printf("Rows: %d\n", rows());
	fflush(stdout);
	#endif

	iSQLCount++;
}

int libpp_postgresql_query_int(char*file, int line, const char *sql)
{
	int iRet;
	PGresult		*result;
	char* res = NULL;
	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	#endif
	
	result=PQexec(db_postgresql, sql);
	if(PQresultStatus(result) == PGRES_BAD_RESPONSE || PQresultStatus(result) == PGRES_FATAL_ERROR)
		sql_failure(file, line, sql, PQresultErrorMessage(result));
	
	if(PQntuples(result)>0) res=PQgetvalue(result,0,0);
	
	if(res&&res[0])
		iRet = atoi(res);
	else
		iRet = 0;

	if (result) PQclear(result);

	iSQLCount++;

	return iRet;
}

const char *libpp_postgresql_query_char(char*file, int line, const char *sql)
{
	static char *sRet = NULL;
	PGresult		*result;
	char* res = NULL;

	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	 #endif

	if (sRet) free(sRet);
	
	result = PQexec(db_postgresql, sql);
	if(PQresultStatus(result) == PGRES_BAD_RESPONSE || PQresultStatus(result) == PGRES_FATAL_ERROR)
		sql_failure(file, line, sql, PQresultErrorMessage(result));

	if(PQntuples(result)>0) res=PQgetvalue(result,0,0);
	if(res&&res[0])
		sRet = strdup(res);
	else
		sRet = NULL;

	if (result) PQclear(result);

	iSQLCount++;

	return sRet;
}


void libpp_postgresql_query_no_return(char*file, int line, const char *sql)
{
	PGresult		*discaud;

	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	#endif

	discaud=PQexec(db_postgresql, sql);
	if(PQresultStatus(discaud) == PGRES_BAD_RESPONSE || PQresultStatus(discaud) == PGRES_FATAL_ERROR)
		sql_failure(file, line, sql, PQresultErrorMessage(discaud));

	iRowsAffected = atoi(PQcmdTuples(discaud));

	if (discaud) {
		PQclear(discaud);
	}


	iSQLCount++;


}



void freeResult(void)
{
	if (query_result) {
		PQclear(query_result);
		query_result = NULL;
	}

}


int rows(void)
{
	if (query_result)
		return PQntuples(query_result);
	else
		return 0;
}

int now_connect(int bTerminate)
{
	if(sql_failure == NULL) sql_failure = pp_postgresql_default_sql_failure;
	//if (!mysql_init (&db_mysql)) {
	//	pp_log(__FILE__,__LINE__, "Cannot initialise MySQL");
	//	sql_failure(__FILE__, __LINE__, "CONNECT", "Cannot initialise MySQL");
	//	exit(EXIT_FAILURE);
	//}
	s_SQL = g_string_sized_new(SQL_SIZE);

	//mysql_options (&db_mysql, MYSQL_READ_DEFAULT_GROUP, "mysqld");

	/* establish a connection to the MySQL server */
   // if (!mysql_real_connect(&db_mysql, 
	//	config_get_value_as_string(CONFIG_ITEM_ID_DB_HOST), 
	//	config_get_value_as_string(CONFIG_ITEM_ID_DB_USER), 
	//	config_get_value_as_string(CONFIG_ITEM_ID_DB_PWD), 
	//	config_get_value_as_string(CONFIG_ITEM_ID_DB_DATABASE), 
	//	db_port, NULL, 0)) {

	db_postgresql=PQsetdbLogin(
		config_get_value_as_string(CONFIG_ITEM_ID_DB_HOST),
		db_port,
		NULL,
		NULL,
		config_get_value_as_string(CONFIG_ITEM_ID_DB_DATABASE),
		config_get_value_as_string(CONFIG_ITEM_ID_DB_USER),
		config_get_value_as_string(CONFIG_ITEM_ID_DB_PWD));
	if(PQstatus(db_postgresql) == CONNECTION_BAD){
		
		g_string_printf(s_SQL, "Could not connect to PostgreSQL using:  Database  = '%s',  Host  = '%s',  Username =  '%s', Password  = '%s'",
			config_get_value_as_string(CONFIG_ITEM_ID_DB_DATABASE), 
			config_get_value_as_string(CONFIG_ITEM_ID_DB_HOST), 
			config_get_value_as_string(CONFIG_ITEM_ID_DB_USER), 
			config_get_value_as_string(CONFIG_ITEM_ID_DB_PWD));

		

		if (bTerminate)	{
			pp_log(__FILE__,__LINE__, "Error %s: %s", s_SQL->str, PQerrorMessage(db_postgresql));
			sql_failure(__FILE__, __LINE__, s_SQL->str, PQerrorMessage(db_postgresql));
			PQfinish(db_postgresql);
			exit(EXIT_FAILURE);
		}
		else
		{	
			PQfinish(db_postgresql);
			return FALSE;
		}

		

	}

	return TRUE;

}

char* getRegistry(const char*host, const char*key, time_t *tm) {
	char *value=NULL;
	g_string_printf(s_SQL, "SELECT rvalue,ctime from perfdata_registry WHERE host = '%s' ", sql_escape(host,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, "AND rkey = '%s'", sql_escape(key,PERCENT_ESCAPE_CHAR));

	query(s_SQL->str);
	currentRow=0;
	if (rows()){
		value = strdup(sData(0));
		if(tm) tm[0] = tData(1);
		freeResult();
	}
	return(value);

}

void setRegistry(const char*host, const char*key, const char*value, time_t tm) {
	char stime[30];
	writeDate(tm, stime);
	g_string_printf(s_SQL, "SELECT count(*) from perfdata_registry WHERE host = '%s' ", sql_escape(host,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, "AND rkey = '%s'", sql_escape(key,PERCENT_ESCAPE_CHAR));
	
	query(s_SQL->str);	

	currentRow=0;
	if (iData(0) > 0) {
		freeResult();
		g_string_printf(s_SQL, "UPDATE perfdata_registry "
				"SET rvalue = '%s', ",sql_escape(value,PERCENT_ESCAPE_CHAR));

		g_string_append_printf(s_SQL, "ctime = '%s' ", stime);
		g_string_append_printf(s_SQL, "WHERE host = '%s' ", sql_escape(host,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, "AND rkey = '%s'", sql_escape(key,PERCENT_ESCAPE_CHAR));

	} else {
		g_string_printf(s_SQL, "INSERT INTO perfdata_registry "
				"(host,rkey, rvalue,ctime) "
				"VALUES ('%s',",sql_escape(host,PERCENT_ESCAPE_CHAR));

		g_string_append_printf(s_SQL, "'%s', ", sql_escape(key,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, "'%s', ", sql_escape(value,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, "'%s') ", stime);

	}
	query_no_return(s_SQL->str);	

}

int addField(char *sTable, char *sField, char *sDescript)
{
	int numberOfRows;
	/* See if the field is there. */
	g_string_printf(s_SQL, "EXPLAIN %s", sTable);

	query(s_SQL->str);
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++) {
		if (!strcmp(sData(0), sField)) return 0;
	}

	/* No field, add */
	g_string_printf(s_SQL, "ALTER TABLE %s ADD %s %s",
		sTable, sField, sDescript);
	query(s_SQL->str);

	return 1;
}

void dropField(char *sTable, char *sField)
{
	int numberOfRows;
	int bHit = 0;

	/* See if the field is there. */
	g_string_printf(s_SQL, "EXPLAIN %s", sTable);

	query(s_SQL->str);
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++) {
		if (!strcmp(sData(0), sField)) {
			bHit = 1;
			break;
		}
	}

	if (!bHit) return;

	/* Field, drop */
	g_string_printf(s_SQL, "ALTER TABLE %s DROP %s",
		sTable, sField);
	query(s_SQL->str);


}


char *sql_escape(const char *sIn,char percent_escape_char)
{
	static char sOut[1024];
	int i, j;

	if (!sIn) return "";

	j = 0;

	for (i = 0; sIn[i] && j < 1023; i++) {
		if (sIn[i] == '\'') sOut[j++] = '\'';
		else if((sIn[i] == '%') && percent_escape_char) sOut[j++] = percent_escape_char;
		else if(sIn[i] == '\\') sOut[j++] = '\\';

		sOut[j++] = sIn[i];

	}

	sOut[j] = 0;


	return sOut;


}
