/*****************************************************************************
 *
 * storage.h - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef __STORAGE__
#define __STORAGE__

#include <time.h>
#include <stdio.h>
#include <libnagios_perfdata_parser.h>
#include <libpp_storage.h>
#include "perfparse-common.h"

typedef struct storage_module_element_t {
	struct storage_module_element_t *prev;
	struct storage_module_element_t *next;
	void * handle;
	storage_module_t *module;
} storage_module_element_t;

int storage_init(const char*argv0);
void storage_load(const char*name);
int storage_connect(void);
int storage_set_registry(const char*hostname, const char*key, const char*value, time_t tm);
int storage_disconnect(void);
int storage_store_line(const log_line_t *log_data);
int storage_store_perf(const log_line_t *log_line, const perf_fields_t *perf_fields);
void storage_atexit(void);
int storage_print_statistics(FILE*fd, void(*print_global_statistics_cb)(FILE*fd, perfparse_statistics_t*stats), perfparse_statistics_t*stats);
int storage_print_usage(FILE*fd, const char*id);
#ifdef PERFPARSED
int storage_periodic_cleanup(void);
void storage_dump_status(void);
#endif

#endif
